#include "scriptinterface.h"

#include "Uni_FleetEngagementText.h"
#include "Uni_FleetEngagementMaps.h"

int32 PlayerDestroyed = 0;
int32 EnemyCaptured = 0;
int32 BonusBalancePoints = 0;
float PlayerHealth = 0.0;
float EnemyHealth = 0.0;
bool PlayerDisengaged = false;

//---------------------------------------------------------------------------------------------
// the following enum is used to identify all non-player teams in the script
// (use kPlayerTeam to access the current players team.)
enum eTeamTypes
{
	// the first 6 team types are reserved for player teams.
	kReserved = kTeam6,

	// add all non-player teams (enemy, allied, monsters, etc...) after this point.
	kEnemyTeam, kAlliedTeam,
};
//---------------------------------------------------------------------------------------------
// called by script interface - 
// should return whatever team is considered to be the primary enemy
int32 tScript::mGetEnemyTeamID (void)
{
	return kEnemyTeam;
}

//---------------------------------------------------------------------------------------------
// returns mission specs used by Dynaverse to build campaigns
void tScript::mScriptSpecs (tMissionSpec* Specs)
{
	Specs->fMissionType = kCampaignMission;
	Specs->fMissionTitle = Messages[kMissionTitle_msg].mCharStr();
	Specs->fSectorType = kEnemyBorder | kWarzone | kOccupiedColonial | kOccupiedCapital | kEnemyCapital | kEnemyColonial;
	Specs->fPlayerRace = kNonPirateMission;
	Specs->fEnemyRace = kNonPirateMission;
	Specs->fNumShips = MissSpecTwoShip | MissSpecThreeShip;
	Specs->fMinBPV = 0;
	Specs->fMaxBPV = kMaxBPV;
	Specs->fMinDate = 10;
	Specs->fMaxDate = 30;
	Specs->fWait = 99;
	Specs->fEnableSupplyDock = kTrue;
}

//---------------------------------------------------------------------------------------------
// called at the start of mission initialization - use to set prestige points 
// and override mission defaults such as:
//		allied race
//		enemy race
//		system name
// and also returns map index to use
int32 tScript::mInitializeStart (tDynaverseScriptInfo& info)
{
	fMission.mSetPrestigePoints( 300, 200, 150, 100, 0 );

	// must set up the races of the teams in the script
	fMission.mSetTeamRace( kPlayerTeam, info.fPlayerRace );
	fMission.mSetTeamRace( kEnemyTeam, info.fEnemyRace );
	fMission.mSetTeamRace( kAlliedTeam, info.fPlayerRace );

	// setup team relations and target goals
	fMission.mSetTeamRelations( kPlayerTeam, kEnemyTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue );
	fMission.mSetTeamRelations( kPlayerTeam, kAlliedTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue );
	fMission.mSetTeamRelations( kEnemyTeam, kAlliedTeam, kTeamAtWar, kGoalDestroy, 1.25, kTrue );
	
	// pick a random map
	return( fMission.mRandomInt32( 1, TotalMaps ));
}

//---------------------------------------------------------------------------------------------
// return kTrue if players ship should be created for this mission
iTruth tScript::mUsePlayerShip (tShipInGame& Ship)
{
	return kTrue;
}

//---------------------------------------------------------------------------------------------
// called to create ships before or during the mission
// Mode is user supplied value (expect for 0, which is initial setup call)
void tScript::mCreateMissionShips (int32 Mode)
{
	tTeamInfo* EnemyTeam = fMission.mGetTeamHandle( kEnemyTeam );
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle( kPlayerTeam );
	tTeamInfo* AlliedTeam = fMission.mGetTeamHandle( kAlliedTeam );

	int32 DiffLev = fMission.mGetDifficultyLevel();

	switch( DiffLev )
	{
		case 0:
			{
				EnemyTeam->mCreateShip( kMaxBPV, kClassDreadnought, 0, kStartPosition_Q );
				EnemyTeam->mCreateShip( kMaxBPV, kClassHeavyCruiser, 0, kStartPosition_R );
				EnemyTeam->mCreateShip( kMaxBPV, kClassLightCruiser, 100, kStartPosition_S );
				EnemyTeam->mCreateShip( kMaxBPV, kClassDestroyer, 100, kStartPosition_T );
				EnemyTeam->mCreateShip( kMaxBPV, kClassFrigate, 100, kStartPosition_U );
			}
			break;

		case 1:
			{
				EnemyTeam->mCreateShip( kMaxBPV, kClassDreadnought, 0, kStartPosition_Q );
				EnemyTeam->mCreateFleet( 2, kMaxBPV, kClassHeavyCruiser, kClassHeavyCruiser, 50, kStartPosition_R );
				EnemyTeam->mCreateShip( kMaxBPV, kClassLightCruiser, 100, kStartPosition_T );
				EnemyTeam->mCreateFleet( 2, kMaxBPV, kClassDestroyer, kClassDestroyer, 100, kStartPosition_U );
				EnemyTeam->mCreateShip( kMaxBPV, kClassFrigate, 100, kStartPosition_W );
			}
			break;

		case 2:
			{
				EnemyTeam->mCreateShip( kMaxBPV, kClassDreadnought, kClassBattleship, 0, kStartPosition_Q );
				EnemyTeam->mCreateShip( kMaxBPV, kClassHeavyBattlecruiser, 0, kStartPosition_R );
				EnemyTeam->mCreateFleet( 3, kMaxBPV, kClassHeavyCruiser, kClassHeavyCruiser, 100, kStartPosition_S );
				EnemyTeam->mCreateShip( kMaxBPV, kClassLightCruiser, 100, kStartPosition_V );
				EnemyTeam->mCreateFleet( 2, kMaxBPV, kClassDestroyer, kClassDestroyer, 100, kStartPosition_W );
				EnemyTeam->mCreateShip( kMaxBPV, kClassFrigate, 100, kStartPosition_Y );
			}
			break;
	}

	// create additional player ships to control for this mission
	PlayerTeam->mCreateFleet( 3, kMaxBPV, kClassHeavyCruiser, kClassHeavyCruiser, 33, kStartPosition_J );

	// create an allied fleet of ships
	AlliedTeam->mCreateShip( kMaxBPV, kClassHeavyCruiser, kClassHeavyBattlecruiser, 0, kStartPosition_M );
	AlliedTeam->mCreateShip( kMaxBPV, kClassLightCruiser, kClassNewHeavyCruiser, 0, kStartPosition_N );
	AlliedTeam->mCreateShip( kMaxBPV, kClassDestroyer, kClassWarDestroyer, 0, kStartPosition_O );
}

//---------------------------------------------------------------------------------------------
// called at the end of mission initialization - use to return briefing message
int32 tScript::mInitializeFinished (void)
{
	// return the index to the briefing message we want to use
	return( kBriefing_msg );
}

//---------------------------------------------------------------------------------------------
// called after all ships have been created and mission is ready to start.
// use to set callback events and change default ship configuration
void tScript::mStartMission (void)
{
	tTeamInfo* EnemyTeam = fMission.mGetTeamHandle( kEnemyTeam );
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle( kPlayerTeam );
	tTeamInfo* AlliedTeam = fMission.mGetTeamHandle( kAlliedTeam );
	tShipInfo* Ship;

	int32 PBPV = PlayerTeam->mGetCombatBPV();
	int32 EBPV = EnemyTeam->mGetCombatBPV();
	BonusBalancePoints = ( 20 * EBPV / PBPV ) - 20;

	// configure player ships
	PlayerTeam->mGetFirstShip()->mSetShipPlayerControlled();
	for( Ship = PlayerTeam->mGetFirstShip(); Ship != NULL; Ship = PlayerTeam->mGetNextShip() )
	{
		Ship->mSetShipThrust( 5 );
	}

	// configure allied ships
	for( Ship = AlliedTeam->mGetFirstShip(); Ship != NULL; Ship = AlliedTeam->mGetNextShip() )
	{
		Ship->mSetShipThrust( 5 );
	}

	for( Ship = EnemyTeam->mGetFirstShip(); Ship != NULL; Ship = EnemyTeam->mGetNextShip() )
	{
		Ship->mSetCommButton( 1, kButtonHail_text );
	}

	fMission.mSetTimedInterval( kTimerLimitSet, 2 );
}

//---------------------------------------------------------------------------------------------
// called whenever a ship is created after a mission has already started
void tScript::mNewMissionShip (tShipInfo* Ship)
{
}

//---------------------------------------------------------------------------------------------
// called during mission when events occur
eGameStatus tScript::mHandleShipEvent (tShipInfo* Ship, iNotifyEvents Event)
{
	tTeamInfo* EnemyTeam = fMission.mGetTeamHandle( kEnemyTeam );
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle( kPlayerTeam );
	tTeamInfo* AlliedTeam = fMission.mGetTeamHandle( kAlliedTeam );

	PlayerHealth = 100.0f - PlayerTeam->mGetTotalDamagePercent();
	EnemyHealth = 100.0f - EnemyTeam->mGetTotalDamagePercent();

	int32 Enemies = EnemyTeam->mGetShipsInPlayCount();

	switch( Event )
	{
		case kNotifyAtTimedInterval:
			fMission.mDisplayMessage( kStarting_msg, kNonCommScriptMessage );
			fMission.mSetTeamRelations( kPlayerTeam, kEnemyTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue );
			fMission.mSetTeamRelations( kPlayerTeam, kAlliedTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue );
			fMission.mSetTeamRelations( kEnemyTeam, kAlliedTeam, kTeamAtWar, kGoalDestroy, 1.25, kTrue );
			break;

		case kNotifyDisengaged:
			if(( Ship->mGetOriginalTeam() == kPlayerTeam ) && ( PlayerTeam->mGetShipsInPlayCount() == 0 ))
			{
				PlayerDisengaged = kTrue;
				fMission.mAddToBonusPrestige( BonusBalancePoints );
				return kPlayerWins;
			}
			break;

		case kNotifyOnCapture:
			fMission.mAddToBonusPrestige( kCaptureBonus );
			EnemyCaptured++;
				
			if ( EnemyTeam->mGetShipsInPlayCount() == 0 )
			{
				fMission.mAddToBonusPrestige( BonusBalancePoints );
				return kPlayerWins;
			}
			break;

		case kNotifyAtDeath:
			if( Ship->mGetOriginalTeam() == kEnemyTeam )
			{
				// see if this is all over.
				if( Enemies == 0 )
				{
					fMission.mAddToBonusPrestige( BonusBalancePoints );
					return kPlayerWins;
				}
			}
			else
			{
				PlayerDestroyed++;
			}
			break;

		case kNotifyCommButtonPress:
			fMission.mDisplayMessage( kStatic_msg, kFleetScriptMessage, Ship );

			for( Ship = EnemyTeam->mGetFirstShip(); Ship != NULL; Ship = EnemyTeam->mGetNextShip() )
			{
				Ship->mClearCommButton( 1 );
			}
			break;
	}
	return fMission.mGetGameStatus();
}

//---------------------------------------------------------------------------------------------
// called when mission has ended but before debriefing and only if player is still alive
eVictoryLevels tScript::mCalcVictoryStatus (void)
{
	tTeamInfo* EnemyTeam = fMission.mGetTeamHandle( kEnemyTeam );
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle( kPlayerTeam );
	tTeamInfo* AlliedTeam = fMission.mGetTeamHandle( kAlliedTeam );

	if( PlayerDisengaged )
	{
		if ( PlayerHealth > EnemyHealth )
		{
			fMission.mAdjustDynaverse( kAdjustWarProgression, +2, EnemyTeam->mGetRace() );
			return kVictory;
		}
		else if ( PlayerHealth < ( EnemyHealth / 2 ) )
		{
			fMission.mAdjustDynaverse( kAdjustWarProgression, -4, EnemyTeam->mGetRace() );
			return kDevastatingDefeat;
		}
		else 
		{
			fMission.mAdjustDynaverse( kAdjustWarProgression, -2, EnemyTeam->mGetRace() );
			return kDefeat;
		}
	}

	if(( PlayerHealth > 90.0 ) || ( EnemyCaptured ))
	{
		fMission.mAdjustDynaverse( kAdjustWarProgression, +4, EnemyTeam->mGetRace() );
		return kAstoundingVictory;
	}

	if( PlayerHealth < 25.0 ) 
	{
		return kDraw;
	}

	// By default
	fMission.mAdjustDynaverse( kAdjustWarProgression, +2, EnemyTeam->mGetRace() );
	return kVictory;

}

//---------------------------------------------------------------------------------------------
// called at end of mission only if player is still alive
void tScript::mGetDebriefingList (eVictoryLevels VictoryLevel)
{
	fMission.mAddToDebriefingList (VictoryLevel + kAstoundingVictory_msg);

	if (PlayerDestroyed == 1)
	{
		fMission.mAddToDebriefingList (kPlayerShipDestroyed_msg);
	}

	if (PlayerDestroyed >= 2)
	{
		fMission.mAddToDebriefingList (kPlayerShipsDestroyed_msg);
	}

	if (EnemyCaptured == 1)
	{
		fMission.mAddToDebriefingList (kCapturedSingleEnemy_msg);
	}

	if (EnemyCaptured >= 2)
	{
		fMission.mAddToDebriefingList (kCapturedMultipleEnemies_msg);
	}
}

//---------------------------------------------------------------------------------------------
// called at the end of every mission
int32 tScript::mGetTechnicalSuccessLevel (tDynaverseScriptInfo& info)
{
	return 0;
}
