// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
//
//  NOTE:	This is a complete production mission. For an explaination of the functions used refer to
//			ScriptInterface.h, ScriptStructs.h and Tst_Examples.cpp. Also see Gen_Template.cpp, 
//			His_Template.cpp, and Mul_Template.cpp for simple template missions.
//
// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<







#include "scriptinterface.h"

#include "His_KobayashiMaruText.h"
#include "His_KobayashiMaruMaps.h"

int32 Cheater = 0;		// 0=non-cheater
int32 KlingonWave = 1;
int32 Chapter = 1;
bool MaruHail = false;
bool NeutralZone = false;
bool KilledAllKlingonsFairly = false;
bool Help = false;
bool PlayerLeft = false;

//---------------------------------------------------------------------------------------------
// the following enum is used to identify all non-player teams in the script
// (use kPlayerTeam to access the current players team.)
enum eTeamTypes
{
	// the first 6 team types are reserved for player teams.
	kReserved = kTeam6,

	// add all non-player teams (enemy, allied, monsters, etc...) after this point.
	kKlingonTeam, kMaruTeam,
};

//---------------------------------------------------------------------------------------------
// called by script interface - 
// should return whatever team is considered to be the primary enemy
int32 tScript::mGetEnemyTeamID (void)
{
	return kPlayerTeam;
}

//---------------------------------------------------------------------------------------------
// returns mission specs used by Dynaverse to build campaigns
void tScript::mScriptSpecs (tMissionSpec* Specs)
{
	Specs->fMissionType = kHistoricalMission;
	Specs->fMissionTitle = Messages[kMissionTitle_msg].mCharStr();
	Specs->fSectorType = kAllSectors;
	Specs->fPlayerRace = kFedMission;
	Specs->fEnemyRace = kAllRaceMission;
	Specs->fNumShips = MissSpecAllShips;
	Specs->fMinBPV = 0;
	Specs->fMaxBPV = kMaxBPV;
	Specs->fMinDate = 9999;
	Specs->fMaxDate = 9999;
	Specs->fWait = 0;
	Specs->fEnableSupplyDock = kDisableAll;
}

//---------------------------------------------------------------------------------------------
// called at the start of mission initialization - use to set prestige points 
// and override mission defaults such as:
//		allied race
//		enemy race
//		system name
// and also returns map index to use
int32 tScript::mInitializeStart (tDynaverseScriptInfo& info)
{
	fMission.mSetTeamRace( kKlingonTeam, kKlingon );
	fMission.mSetTeamRace( kMaruTeam, kFederation );

	fMission.mSetTeamRelations( kPlayerTeam, kMaruTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue );
	fMission.mSetTeamRelations( kPlayerTeam, kKlingonTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue );
	fMission.mSetTeamRelations( kMaruTeam, kKlingonTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue );

	// read the INI file for the cheat code (0=non-cheater)
	Cheater = fMission.mGetINIFileInt (kINIFileHeader_msg, kINIFileLabel_msg);

	// pick a map
	return 1;
}

//---------------------------------------------------------------------------------------------
// return kTrue if players ship should be created for this mission
iTruth tScript::mUsePlayerShip (tShipInGame& Ship)
{
	return kFalse;
}

//---------------------------------------------------------------------------------------------
// called to create ships before or during the mission
// Mode is user supplied value (expect for 0, which is initial setup call)
void tScript::mCreateMissionShips (int32 Mode)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* KlingonTeam = fMission.mGetTeamHandle (kKlingonTeam);
	tTeamInfo* MaruTeam = fMission.mGetTeamHandle (kMaruTeam);

	int32 OldX = 0;
	int32 OldY = 0;
	int32 NewX = 0;
	int32 NewY = 0;

	if( Mode == 0 )
	{
		// player's ship
		if( Cheater == 0 )
		{
			PlayerTeam->mCreateShip( "F-CA+", kStartPosition_G );
		}
		else
		{
			PlayerTeam->mCreateShip( "F-BB", kStartPosition_G );
		}

		// Kobayashi Maru
		MaruTeam->mCreateShip( kMaxBPV, kClassFreighter, 0, kStartPosition_K );
	}
	else if( Mode == 1 )	// Klingon Wave #1: 3 ships
	{
		PlayerTeam->mGetFirstShip()->mGetGridPosition( &OldX, &OldY );
		NewX = OldX + 2;
		NewY = OldY + 5;
		KlingonTeam->mCreateFleet( 3, kMaxBPV, kClassHeavyCruiser, kClassHeavyCruiser, 0, kStartPosition_Specified, NewX, NewY );
	}
	else if( Mode == 2 )	// Klingon Wave #2: 6 ships
	{
		PlayerTeam->mGetFirstShip()->mGetGridPosition( &OldX, &OldY );
		NewX = OldX - 2;
		NewY = OldY - 5;
		KlingonTeam->mCreateFleet( 3, kMaxBPV, kClassHeavyCruiser, kClassHeavyCruiser, 0, kStartPosition_Specified, NewX, NewY );
		NewX = OldX + 2;
		NewY = OldY + 5;
		KlingonTeam->mCreateFleet( 3, kMaxBPV, kClassHeavyCruiser, kClassHeavyCruiser, 0, kStartPosition_Specified, NewX, NewY );
	}
	else if( Mode == 3 )	// Wave #3: 9 ships
	{
		PlayerTeam->mGetFirstShip()->mGetGridPosition( &OldX, &OldY );
		NewX = OldX - 2;
		NewY = OldY - 5;
		KlingonTeam->mCreateFleet( 3, kMaxBPV, kClassHeavyCruiser, kClassHeavyCruiser, 0, kStartPosition_Specified, NewX, NewY );
		NewX = OldX + 2;
		NewY = OldY + 5;
		KlingonTeam->mCreateFleet( 3, kMaxBPV, kClassHeavyCruiser, kClassHeavyCruiser, 0, kStartPosition_Specified, NewX, NewY );
		NewX = OldX - 5;
		NewY = OldY + 2;
		KlingonTeam->mCreateFleet( 3, kMaxBPV, kClassHeavyCruiser, kClassHeavyCruiser, 0, kStartPosition_Specified, NewX, NewY );
	}
	else if( Mode == 4 )	// Final Wave #4: 12 ships
	{
		PlayerTeam->mGetFirstShip()->mGetGridPosition( &OldX, &OldY );
		NewX = OldX - 2;
		NewY = OldY - 5;
		KlingonTeam->mCreateFleet( 3, kMaxBPV, kClassHeavyCruiser, kClassHeavyCruiser, 0, kStartPosition_Specified, NewX, NewY );
		NewX = OldX + 2;
		NewY = OldY + 5;
		KlingonTeam->mCreateFleet( 3, kMaxBPV, kClassHeavyCruiser, kClassHeavyCruiser, 0, kStartPosition_Specified, NewX, NewY );
		NewX = OldX - 5;
		NewY = OldY + 2;
		KlingonTeam->mCreateFleet( 3, kMaxBPV, kClassHeavyCruiser, kClassHeavyCruiser, 0, kStartPosition_Specified, NewX, NewY );
		NewX = OldX - 5;
		NewY = OldY - 2;
		KlingonTeam->mCreateFleet( 3, kMaxBPV, kClassHeavyCruiser, kClassHeavyCruiser, 0, kStartPosition_Specified, NewX, NewY );
	}
}

//---------------------------------------------------------------------------------------------
// called at the end of mission initialization - use to return briefing message
int32 tScript::mInitializeFinished (void)
{
	// return the index to the briefing message we want to use
	return (kBriefing_msg);
}

//---------------------------------------------------------------------------------------------
// called after all ships have been created and mission is ready to start.
// use to set callback events and change default ship configuration
void tScript::mStartMission (void)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* KlingonTeam = fMission.mGetTeamHandle (kKlingonTeam);
	tTeamInfo* MaruTeam = fMission.mGetTeamHandle (kMaruTeam);
	tShipInfo* Ship;

	int32 MaruX = 0;
	int32 MaruY = 0;

	MaruTeam->mGetFirstShip()->mGetGridPosition (&MaruX, &MaruY);

	Ship = PlayerTeam->mGetFirstShip();
	Ship->mSetShipPlayerControlled();
	Ship->mChangeName( kEnterprise_name );
	Ship->mSetShipThrust (10);
	Ship->mTrackTacticalDistance( 80.0, MaruX, MaruY );
	Ship->mSetShipCallbackEvents( kNotifyBeginTractor );

	Ship = MaruTeam->mGetFirstShip();
	Ship->mChangeName( kKobayashiMaru_name );
	Ship->mDamageSystem( kDamageWeaponsSystemsAndShields, 75.0 );
	Ship->mSetShipAIControlled( kFalse );

	fMission.mDisplayMessage( kStarting_msg );
	fMission.mSetTimedInterval ( kTimerLimitSet, 8 );
}

//---------------------------------------------------------------------------------------------
// called whenever a ship is created after a mission has already started
void tScript::mNewMissionShip (tShipInfo* Ship)
{
	Ship->mSetCommButton( 1, kButtonHail_text );

	float DamagePoints = 40.0 + ( 10.0 * KlingonWave );

	if( Cheater == 1 )
	{
		Ship->mDamageSystem( kDamageMaxWeapons, 25.0 );
		Ship->mDamageSystem( kDamageInternalsWithPoints, DamagePoints );
		Ship->mDamageSystem( kDamageShield_4_Aft, 100.0 );
		Ship->mDamageSystem( kDamageDamageControlGroupSystem, 100.0 );
	}
}

//---------------------------------------------------------------------------------------------
// called during mission when events occur
eGameStatus tScript::mHandleShipEvent (tShipInfo* Ship, iNotifyEvents Event)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tTeamInfo* KlingonTeam = fMission.mGetTeamHandle (kKlingonTeam);
	tTeamInfo* MaruTeam = fMission.mGetTeamHandle (kMaruTeam);
	tShipInfo* Target = fMission.mGetShipTarget( fPlayerShip );

	int32 MaruX = 0;
	int32 MaruY = 0;

	switch( Event )
	{
		case kNotifyBeginTractor:
			Ship = PlayerTeam->mGetFirstShip();
			Ship->mSetShipCallbackEvents( kNotifyBeginTractor );

			if( Target == MaruTeam->mGetFirstShip() )
			{
				Ship->mDamageSystem( kDamageTractorGroupSystem, 100.0 );
				fMission.mDisplayMessage( kTractorsDamaged_msg );
			}
			break;

		case kNotifyCommButtonPress:
			switch( fEventID )
			{
				case kButtonHail_text:
					if( Ship->mGetTeam() == kKlingonTeam )
					{
						fMission.mDisplayMessage( kHailKlingon_msg );
					}
					else
					{
						if( MaruHail == false )
						{
							fMission.mDisplayMessage( kHailMaruFirst_msg );
							MaruHail = true;
							Chapter = 4;
							fMission.mSetTimedInterval( kTimerLimitSet_2, 3 );
							fMission.mSetTimedInterval( kTimerLimitSet, 8 );
						}
						else
						{
							fMission.mDisplayMessage( kHailMaruAgain_msg );
						}
					}
					break;

				case kButtonYes_text:
					Ship->mClearCommButton( 2 );
					fMission.mDisplayMessage( kPositiveResponse_msg );
					Help = true;
					break;

				case kButtonNo_text:
					Ship->mClearCommButton( 1 );
					fMission.mDisplayMessage( kNegativeResponse_msg );
					return kPlayerLoses;
			}
			break;

		case kNotifyAtTimedInterval:
			switch( fEventID )
			{
				case kTimerTurnsSet:
					mCreateMissionShips( KlingonWave );
					fMission.mDisplayMessage( ( kKlingonsSir3_msg + KlingonWave - 1 ), kScienceScriptMessage );

					if( Cheater == 1 )
					{
						Ship = PlayerTeam->mGetFirstShip();
						Ship->mAccessSupplies( kSetSupplyCount, kSupplyScatterPack, 10 );
						Ship->mAccessSupplies( kSetSupplyCount, kSupplySuicides, 10 );
						Ship->mAccessSupplies( kSetSupplyCount, kSupplyMines, 20 );
					}
					break;

				case kTimerLimitSet:
					switch( Chapter )
					{
						case 1:
							fMission.mDisplayMessage( kHelmLeaving_msg );
							fMission.mSetTimedInterval( kTimerLimitSet, 7 );
							Chapter++;
							break;

						case 2:
							fMission.mDisplayMessage( kDistressSignal_msg );
							fMission.mSetTimedInterval( kTimerLimitSet, 7 );
							Chapter++;
							break;

						case 3:
							fMission.mDisplayMessage( kDistressCall_msg );
							MaruTeam->mGetFirstShip()->mSetCommButton( 1, kButtonHail_text );
							Chapter++;
							break;

						case 4:
							fMission.mDisplayMessage( kSecondHail_msg );
							Ship = MaruTeam->mGetFirstShip();
							Ship->mSetCommButton( 1, kButtonYes_text );
							Ship->mSetCommButton( 2, kButtonNo_text );
							break;
					}
					break;

				case kTimerLimitSet_2:
					fMission.mDisplayMessage( kGammaHydra10_msg, kNonCommScriptMessage );
					break;
			}
			break;

		case kNotifyAtDeath:
			if( Ship->mGetTeam() == kPlayerTeam )
			{
				return kPlayerLoses;
			}
			else if( Ship->mGetTeam() == kMaruTeam )
			{
				fMission.mDisplayMessage( kMaruDestroyed_msg, kScienceScriptMessage );
			}
			else if( Ship->mGetTeam() == kKlingonTeam )
			{
				if( Cheater == 1)
				{
					PlayerTeam->mGetFirstShip()->mDamageSystem( kDamageMaxShields, 0.0 );
				}

				if( KlingonTeam->mGetShipsInPlayCount() == 0 )
				{
					KlingonWave++;
					if( KlingonWave == 5 )
					{
						KilledAllKlingonsFairly = true;
						return kPlayerWins;
					}
					fMission.mSetTimedInterval( kTimerTurnsSet, 1 );
				}
			}
			break;

		case kNotifyOnCapture:
			if( Ship->mGetOriginalTeam() == kKlingonTeam )
			{
				Ship->mDamageSystem( kDamageInternalsWithPoints, 5000 );
				fMission.mDisplayMessage( kKlingonBoom_msg, kScienceScriptMessage );
			}
			break;

		case kNotifyAtDistance:
			if( Ship->mGetTrackingDistance() == 80.0 )
			{
				MaruTeam->mGetFirstShip()->mGetGridPosition ( &MaruX, &MaruY );
				PlayerTeam->mGetFirstShip()->mTrackTacticalDistance( 60.0, MaruX, MaruY );
				fMission.mDisplayMessage( kApproachingZone_msg );
			}
			else
			{
				fMission.mDisplayMessage( kEnteringZone_msg );
				NeutralZone = true;
				fMission.mSetTimedInterval( kTimerTurnsSet, 1 );
			}
			break;

		case kNotifyDisengaged:
			if(( Ship->mGetOriginalTeam() == kPlayerTeam ) && ( PlayerTeam->mGetShipsInPlayCount() == 0 ))
			{
				PlayerLeft = true;
				return kPlayerWins;
			}
			break;
	}

	return fMission.mGetGameStatus();
}

//---------------------------------------------------------------------------------------------
// called when mission has ended but before debriefing and only if player is still alive
eVictoryLevels tScript::mCalcVictoryStatus (void)
{
	return (fMission.mGetVictoryLevel (0, 0));
}

//---------------------------------------------------------------------------------------------
// called at end of mission only if player is still alive
void tScript::mGetDebriefingList (eVictoryLevels VictoryLevel)
{
	if(( Help == false ) || ( PlayerLeft == true ))
	{
		fMission.mAddToDebriefingList( kDefeatNoHelp_msg );
	}
	else if( KilledAllKlingonsFairly == true )
	{
		fMission.mAddToDebriefingList (kVictory_msg);
	}
	else
	{
		fMission.mAddToDebriefingList (kDefeat_msg);
	}
}

//---------------------------------------------------------------------------------------------
// called at the end of every mission
int32 tScript::mGetTechnicalSuccessLevel (tDynaverseScriptInfo& info)
{
	return 0;
}

