// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
//
//  NOTE:	This is a complete production mission. For an explaination of the functions used refer to
//			ScriptInterface.h, ScriptStructs.h and Tst_Examples.cpp. Also see Gen_Template.cpp, 
//			His_Template.cpp, and Mul_Template.cpp for simple template missions.
//
// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<







#include "scriptinterface.h"

#include "His_TerrorText.h"
#include "His_TerrorMaps.h"

int32 DiffLevel = 0;
int32 DeadOutposts = 0;
tShipInfo* BlowThisOutpost = NULL;
bool Hunting = false;
bool EnterpriseDestroyed = false;
bool EnterpriseCaptured = false;
bool PlayerCrippled = false;
bool PlayerDestroyed = false;
bool GoodDisengage = false;
bool CanTrick = false;
bool DebrisTrick = false;
bool CapturedOutpost = false;

enum eOutpost
{
	Outpost1 = 0,
	Outpost2,
	Outpost3,
	Outpost4,
	Outpost5,
	Outpost6,
	Outpost7,
	Outpost8,
};

//---------------------------------------------------------------------------------------------
// the following enum is used to identify all non-player teams in the script
// (use kPlayerTeam to access the current players team.)
enum eTeamTypes
{
	// the first 6 team types are reserved for player teams.
	kReserved = kTeam6,

	// add all non-player teams (enemy, allied, monsters, etc...) after this point.
	kEnterpriseTeam, kOutpostTeam, kAllyTeam,
};

//---------------------------------------------------------------------------------------------
// called by script interface - 
// should return whatever team is considered to be the primary enemy
int32 tScript::mGetEnemyTeamID (void)
{
	return kPlayerTeam;
}

//---------------------------------------------------------------------------------------------
// returns mission specs used by Dynaverse to build campaigns
void tScript::mScriptSpecs (tMissionSpec* Specs)
{
	Specs->fMissionType = kHistoricalMission;
	Specs->fMissionTitle = Messages[kMissionTitle_msg].mCharStr();
	Specs->fSectorType = kAllSectors;
	Specs->fPlayerRace = kRomulanMission;
	Specs->fEnemyRace = kAllRaceMission;
	Specs->fNumShips = MissSpecAllShips;
	Specs->fMinBPV = 0;
	Specs->fMaxBPV = kMaxBPV;
	Specs->fMinDate = 9999;
	Specs->fMaxDate = 9999;
	Specs->fWait = 0;
	Specs->fEnableSupplyDock = kDisableAll;
}

//---------------------------------------------------------------------------------------------
// called at the start of mission initialization - use to set prestige points 
// and override mission defaults such as:
//		allied race
//		enemy race
//		system name
// and also returns map index to use
int32 tScript::mInitializeStart (tDynaverseScriptInfo& info)
{
	DiffLevel = fMission.mGetDifficultyLevel();

	fMission.mSetTeamRace( kPlayerTeam, info.fPlayerRace );
	fMission.mSetTeamRace( kAllyTeam, info.fPlayerRace );
	fMission.mSetTeamRace( kEnterpriseTeam, kFederation );
	fMission.mSetTeamRace( kOutpostTeam, kFederation );

	fMission.mSetTeamRelations( kPlayerTeam, kEnterpriseTeam, kTeamHostile, kGoalIgnore, 1.0, kTrue );
	fMission.mSetTeamRelations( kPlayerTeam, kAllyTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue );
	fMission.mSetTeamRelations( kPlayerTeam, kOutpostTeam, kTeamHostile, kGoalIgnore, 1.0, kTrue );
	fMission.mSetTeamRelations( kEnterpriseTeam, kAllyTeam, kTeamHostile, kGoalIgnore, 1.0, kTrue );
	fMission.mSetTeamRelations( kOutpostTeam, kAllyTeam, kTeamAtWar, kGoalIgnore, 1.0, kTrue );
	fMission.mSetTeamRelations( kOutpostTeam, kEnterpriseTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue );

	// pick a map
	return 1;
}

//---------------------------------------------------------------------------------------------
// return kTrue if players ship should be created for this mission
iTruth tScript::mUsePlayerShip (tShipInGame& Ship)
{
	return kFalse;
}

//---------------------------------------------------------------------------------------------
// called to create ships before or during the mission
// Mode is user supplied value (expect for 0, which is initial setup call)
void tScript::mCreateMissionShips (int32 Mode)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle( kPlayerTeam );
	tTeamInfo* EnterpriseTeam = fMission.mGetTeamHandle( kEnterpriseTeam );
	tTeamInfo* OutpostTeam = fMission.mGetTeamHandle( kOutpostTeam );
	tTeamInfo* AllyTeam = fMission.mGetTeamHandle( kAllyTeam );

	if( Mode == 0 )
	{
		int32 DistMod = 6;

		if( DiffLevel == 2 )
		{
			DistMod = 3;
		}

		for( int32 Loop = 0; Loop < 8; Loop++ )
		{
			if( fMission.mRandomInt32( 0, 4 ) <= DiffLevel )
			{
				OutpostTeam->mCreateShip( "N-ABDr", kStartPosition_Specified, 14, ( 48 - ( DistMod * Loop )));
			}
			else
			{
				OutpostTeam->mCreateShip( "N-ABPh", kStartPosition_Specified, 15, ( 48 - ( DistMod * Loop )));
			}
		}

		if( DiffLevel == 2 )	// hard difficulty level
		{
			PlayerTeam->mCreateShip( "R-WB+", kStartPosition_G );
		}
		else
		{
			PlayerTeam->mCreateShip( "R-WER", kStartPosition_G );
		}
	}
	
	if( Mode == 1 )
	{
		int32 NewY = 0;
		int32 NewX = 0;
		int32 MapX = 0;
		int32 MapY = 0;
		
		fPlayerShip->mGetGridPosition( &NewX, &NewY );
		fMission.mGetMapDimensions( &MapX, &MapY );

		NewY += fMission.mRandomInt32( 2, 6 );

		if( NewY > MapY )
		{
			NewY = MapY;
		}

		EnterpriseTeam->mCreateShip( "F-CA", kStartPosition_Specified, 1, NewY );
	}

	if( Mode == 2 )
	{
		AllyTeam->mCreateShip( "R-FQS", kStartPosition_H );
	}
}

//---------------------------------------------------------------------------------------------
// called at the end of mission initialization - use to return briefing message
int32 tScript::mInitializeFinished (void)
{
	// return the index to the briefing message we want to use
	if( DiffLevel == 2 )
	{
		return( kBriefingHard_msg );
	}

	return( kBriefing_msg );
}

//---------------------------------------------------------------------------------------------
// called after all ships have been created and mission is ready to start.
// use to set callback events and change default ship configuration
void tScript::mStartMission (void)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle( kPlayerTeam );
	tTeamInfo* EnterpriseTeam = fMission.mGetTeamHandle( kEnterpriseTeam );
	tTeamInfo* OutpostTeam = fMission.mGetTeamHandle( kOutpostTeam );
	tTeamInfo* AllyTeam = fMission.mGetTeamHandle( kAllyTeam );
	tShipInfo* Ship;

	int32 OutpostName = 0;
	int32 OutpostHealthModifier = 90 - ( DiffLevel * 15 );
	int32 DiffTurns = 15 - ( DiffLevel * 3 );

	// configure the player's ship
	Ship = PlayerTeam->mGetFirstShip();
	Ship->mSetShipPlayerControlled();
	Ship->mChangeName( kRomulan_name );
	Ship->mSetShipCallbackEvents( kNotifyDamage );
	Ship->mSetOfficerRank( kEngineeringOfficer, kVeteran );
	Ship->mSetOfficerRank( kScienceOfficer, kVeteran );
	Ship->mSetOfficerRank( kHelmOfficer, kVeteran );
	Ship->mSetOfficerRank( kWeaponsOfficer, kVeteran );
	Ship->mSetOfficerRank( kSecurityOfficer, kVeteran );

	// configure the Outposts
	for( Ship = OutpostTeam->mGetFirstShip(); Ship != NULL; Ship = OutpostTeam->mGetNextShip() )
	{
		Ship->mSetCommButton( 1, kButtonHail_text );
		Ship->mChangeName( kOutpost1_name + OutpostName );
		Ship->mSetOfficerRank( kWeaponsOfficer, kSenior );
		Ship->mDamageSystem( kDamageForwardHullSystem, OutpostHealthModifier );
		Ship->mDamageSystem( kDamageCenterHullSystem, OutpostHealthModifier );
		Ship->mDamageSystem( kDamageAftHullSystem, OutpostHealthModifier );
		Ship->mDamageSystem( kDamageArmorSystem, 100.0 );
		Ship->mChangeAIOfficerState( kECCMOfficer, kFalse );
		Ship->mAccessSupplies( kSetSupplyCount, kSupplyMarines, 3 );
		OutpostName++;

		if( DiffLevel == 0 )	// easy
		{
			Ship->mChangeAIOfficerState( kRepairOfficer, kFalse );
		}
	}

	fMission.mSetTimedInterval( kTimerLimitSet, 1 );			// starting messages
	fMission.mSetTimedInterval( kTimerTurnsSet, DiffTurns );	// Enterprise shows up
	fMission.mSetTimedInterval( kTimerTurnsSet_5, 12 );			// Romulan freighter
}

//---------------------------------------------------------------------------------------------
// called whenever a ship is created after a mission has already started
void tScript::mNewMissionShip (tShipInfo* Ship)
{
	if( Ship->mGetTeam() == kEnterpriseTeam )
	{
		// configure the Enterprise
		Ship->mChangeName( kEnterprise_name );
		Ship->mSetShipCallbackEvents( kNotifyDamage );
		Ship->mSetFinalDestination();
		Ship->mSetCommButton( 1, kButtonHail_text );
		Ship->mSetShipMaxSpeed( 16 );

		if( DiffLevel == 0 )
		{
			Ship->mChangeAIOfficerState( kRepairOfficer, kFalse );
			Ship->mChangeAIOfficerState( kECCMOfficer, kFalse );
			Ship->mChangeAIOfficerState( kECMOfficer, kFalse );
			Ship->mChangeAIOfficerState( kTBombOfficer, kFalse );
		}
	}

	if( Ship->mGetTeam() == kAllyTeam )
	{
		Ship->mSetCommButton( 1, kButtonHail_text );
	}
}

//---------------------------------------------------------------------------------------------
// called during mission when events occur
eGameStatus tScript::mHandleShipEvent (tShipInfo* Ship, iNotifyEvents Event)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle( kPlayerTeam );
	tTeamInfo* EnterpriseTeam = fMission.mGetTeamHandle( kEnterpriseTeam );
	tTeamInfo* OutpostTeam = fMission.mGetTeamHandle( kOutpostTeam );
	tTeamInfo* AllyTeam = fMission.mGetTeamHandle( kAllyTeam );

	switch( Event )
	{
		case kNotifyAtTimedInterval:
			switch( fEventID )
			{
				case kTimerLimitSet:
					fMission.mDisplayMessage( kStarting_msg, kNonCommScriptMessage );
					fMission.mSetTeamRelations( kPlayerTeam, kEnterpriseTeam, kTeamHostile, kGoalIgnore, 1.0, kTrue );
					fMission.mSetTeamRelations( kPlayerTeam, kAllyTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue );
					fMission.mSetTeamRelations( kPlayerTeam, kOutpostTeam, kTeamHostile, kGoalIgnore, 1.0, kTrue );
					fMission.mSetTeamRelations( kEnterpriseTeam, kAllyTeam, kTeamHostile, kGoalIgnore, 1.0, kTrue );
					fMission.mSetTeamRelations( kOutpostTeam, kAllyTeam, kTeamAtWar, kGoalIgnore, 1.0, kTrue );
					fMission.mSetTeamRelations( kOutpostTeam, kEnterpriseTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue );
					break;

				case kTimerLimitSet_2:	// Enterprise close enough to player
					Ship = EnterpriseTeam->mGetFirstShip();
					
					if( Ship != NULL )
					{
						if( fPlayerShip->mGetTacticalDistance( Ship ) < 30 )
						{
							fMission.mDisplayMessage( kEntApproaching_msg, kScienceScriptMessage );
						}
						else
						{
							fMission.mSetTimedInterval( kTimerLimitSet_2, 4 );
						}
					}
					break;

				case kTimerLimitSet_3:	// about to pull a debris trick
					if(( CanTrick == true ) && ( PlayerCrippled == false ))
					{
						fMission.mDisplayMessage( kEventDebris_msg );
						fMission.mSetTimedInterval( kTimerTurnsSet_3, 1 );
					}
					break;

				case kTimerLimitSet_4:	// player has been crippled and is about to blow up
					if( fPlayerShip != NULL )
					{
						fMission.mDisplayMessage( kKirkSpeech1_msg );
						fMission.mSetTimedInterval( kTimerLimitSet_5, 10 );
					}
					break;

				case kTimerLimitSet_5:
					if( fPlayerShip != NULL )
					{
						fMission.mDisplayMessage( kKirkSpeech2_msg );
						fMission.mSetTimedInterval( kTimerLimitSet_6, 10 );
					}
					break;

				case kTimerLimitSet_6:
					if( fPlayerShip != NULL )
					{
						fMission.mDisplayMessage( kKirkSpeech3_msg );
						fMission.mSetTimedInterval( kTimerLimitSet_7, 10 );
					}
					break;

				case kTimerLimitSet_7:
					if( fPlayerShip != NULL )
					{
						fMission.mDisplayMessage( kKirkSpeech4_msg );
						fMission.mSetTimedInterval( kTimerLimitSet_8, 10 );
					}
					break;

				case kTimerLimitSet_8:
					if( fPlayerShip != NULL )
					{
						fPlayerShip->mDamageSystem( kDamageExcessDamageSystem, 100.0 );
						fPlayerShip->mDamageSystem( kDamageInternalsWithPoints, 5000 );
					}
					break;

				case kTimerTurnsSet:	// Enterprise arrival
					mCreateMissionShips( 1 );
					fMission.mDisplayMessage( kEntArriving_msg, kScienceScriptMessage );
					fMission.mSetTimedInterval( kTimerLimitSet_2, 4 );
					break;

				case kTimerTurnsSet_2:	// Captured outpost explodes
					if( BlowThisOutpost != NULL )
					{
						Ship = BlowThisOutpost;

						if( Ship->mTestShipStatus( kDestroyed ) == false )
						{
							Ship->mDamageSystem( kDamageExcessDamageSystem, 100.0 );
							Ship->mDamageSystem( kDamageInternalsWithPoints, 5000 );
							BlowThisOutpost = NULL;
						}
					}
					break;

				case kTimerTurnsSet_3:
					Hunting = false;
					DebrisTrick = true;

					Ship = EnterpriseTeam->mGetFirstShip();

					if( Ship != NULL )
					{
						int32 NewX = 0;
						int32 NewY = 0;

						fPlayerShip->mGetGridPosition( &NewX, &NewY );
						NewX = NewX - fMission.mRandomInt32( -1, 1 );
						NewY = NewY - fMission.mRandomInt32( 1, 3 );
						Ship->mSetFinalDestination( NewX, NewY );

						fMission.mDisplayMessage( kEventEntFooled_msg );
						fMission.mSetTeamRelations( kPlayerTeam, kEnterpriseTeam, kTeamHostile, kGoalIgnore, 1.0, kTrue );
						fMission.mSetTimedInterval( kTimerTurnsSet_4, 5 );
					}

					if( fPlayerShip != NULL )
					{
						fPlayerShip->mSetShipCallbackEvents( kNotifyDamage );
					}
					break;

				case kTimerTurnsSet_4:
					Ship = EnterpriseTeam->mGetFirstShip();

					if( Ship != NULL )
					{
						int32 RandomTurns = 0;
						Ship->mClearCommands();

						if( Hunting == false )
						{
							Hunting = true;
							fMission.mSetTeamRelations( kPlayerTeam, kEnterpriseTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue );
							Ship->mSetShipTarget( fPlayerShip, kAbsolutePriority );
							Ship->mSetShipMaxSpeed( 24 );
							RandomTurns = fMission.mRandomInt32( 5, 7 );
						}
						else
						{
							int32 NewX = 0;
							int32 NewY = 0;
							Hunting = false;
							fMission.mDisplayMessage( fMission.mRandomInt32( kEntBreak1_msg, kEntBreak3_msg ), kScienceScriptMessage );
							fMission.mSetTeamRelations( kPlayerTeam, kEnterpriseTeam, kTeamAtWar, kGoalIgnore, 1.0, kTrue );

							fPlayerShip->mGetGridPosition( &NewX, &NewY );
							NewX = NewX - fMission.mRandomInt32( -1, 1 );
							NewY = NewY - fMission.mRandomInt32( 1, 3 );
							Ship->mSetFinalDestination( NewX, NewY );
							Ship->mSetShipMaxSpeed( 16 );

							RandomTurns = fMission.mRandomInt32( 1, 3 );
						}

						fMission.mSetTimedInterval( kTimerTurnsSet_4, RandomTurns );
					}
					break;

				case kTimerTurnsSet_5:
					mCreateMissionShips( 2 );	// Romulan freighter
					fMission.mDisplayMessage( kAllyArrived_msg, kScienceScriptMessage );
					break;

				case kTimerTurnsSet_6:	// parts
					Ship = AllyTeam->mGetFirstShip();

					if( Ship != NULL )
					{
						if( Ship->mGetTacticalDistance( fPlayerShip ) > 5 )
						{
							fMission.mDisplayMessage( kAllyNoTransfer_msg, kFleetScriptMessage );
							Ship->mSetCommButton( 1, kButtonTransferParts_text );
							Ship->mSetCommButton( 2, kButtonTransferSupplies_text );
						}
						else
						{
							Ship = PlayerTeam->mGetFirstShip();

							if( Ship != NULL )
							{
								Ship->mAccessSupplies( kSetSupplyCount, kSupplySpareParts, 6 );
							}

							fMission.mSetTimedInterval( kTimerTurnsSet_8, 1 );
						}
					}
					break;

				case kTimerTurnsSet_7:	// supplies
					Ship = AllyTeam->mGetFirstShip();

					if( Ship != NULL )
					{
						if( Ship->mGetTacticalDistance( fPlayerShip ) > 5 )
						{
							fMission.mDisplayMessage( kAllyNoTransfer_msg, kFleetScriptMessage );
							Ship->mSetCommButton( 1, kButtonTransferParts_text );
							Ship->mSetCommButton( 2, kButtonTransferSupplies_text );
						}
						else
						{
							Ship = PlayerTeam->mGetFirstShip();

							if( Ship != NULL )
							{
								int32 PlayerMarines = Ship->mAccessSupplies( kGetSupplyCount, kSupplyMarines );
								PlayerMarines += 9;
								Ship->mAccessSupplies( kSetSupplyCount, kSupplyMarines, PlayerMarines );

								int32 PlayerMines = Ship->mAccessSupplies( kGetSupplyCount, kSupplyMines );
								PlayerMines += 5;
								Ship->mAccessSupplies( kSetSupplyCount, kSupplyMines, PlayerMines );

								fMission.mSetTimedInterval( kTimerTurnsSet_8, 1 );
							}
						}
					}
					break;

				case kTimerTurnsSet_8:	// ally freighter disengaging
					Ship = AllyTeam->mGetFirstShip();

					if( Ship != NULL )
					{
						fMission.mDisplayMessage( kAllyLeaving_msg, kFleetScriptMessage, Ship );
						Ship->mDisengage( kByMapEdge, kAbsolutePriority );
					}
					break;
			}
			break;

		case kNotifyAtDeath:
			if( Ship->mGetOriginalTeam() == kPlayerTeam)
			{
				PlayerDestroyed = true;
				return kPlayerLoses;
			}
			
			if( Ship->mGetOriginalTeam() == kOutpostTeam )
			{
				DeadOutposts++;
				fMission.mSetTeamRelations( kPlayerTeam, kEnterpriseTeam, kTeamAtWar, kGoalDestroy, 1.0, kTrue );

				if( OutpostTeam->mGetShipsInPlayCount() == 0 )
				{
					fMission.mDisplayMessage( kAllOutpostsDead_msg, kScienceScriptMessage );
				}
				else if( OutpostTeam->mGetShipsInPlayCount() == 4 )
				{
					fMission.mDisplayMessage( kHalfOutpostsDead_msg, kScienceScriptMessage );
				}
				else if( fMission.mRandomInt32( 1, 2 ) == 1 )
				{
					fMission.mDisplayMessage( fMission.mRandomInt32( kOutpostDestroyed1_msg, kOutpostDestroyed3_msg ), kFleetScriptMessage );
				}
			}

			if( Ship->mGetOriginalTeam() == kEnterpriseTeam )
			{
				EnterpriseDestroyed = true;
				fMission.mDisplayMessage( kEntDestroyed_msg, kScienceScriptMessage );
			}

			if( Ship->mGetOriginalTeam() == kAllyTeam )
			{
				fMission.mDisplayMessage( kAllyDestroyed_msg, kScienceScriptMessage );
			}
			break;

		case kNotifyOnCapture:
			if( Ship->mGetOriginalTeam() == kEnterpriseTeam )
			{
				fMission.mDisplayMessage( kEntCaptured_msg );
				EnterpriseCaptured = true;
			}

			if( Ship->mGetOriginalTeam() == kOutpostTeam )
			{
				fMission.mDisplayMessage( kOutpostCaptured_msg, kScienceScriptMessage );
				fMission.mSetTimedInterval( kTimerTurnsSet_2, 2 );
				BlowThisOutpost = Ship;
				CapturedOutpost = true;
			}
			break;

		case kNotifyDamage:
			if( Ship->mGetTeam() == kPlayerTeam )
			{
				if( DebrisTrick == false )
				{
					if( Ship->mGetDamagePercent() > 50 )
					{
						CanTrick = true;
						fMission.mSetTimedInterval( kTimerLimitSet_3, 4 );
					}
					else
					{
						Ship->mSetShipCallbackEvents( kNotifyDamage );
					}
				}
				else
				{
					if( Ship->mGetDamagePercent() > 90 )
					{
						PlayerCrippled = true;
						fMission.mSetTeamRelations( kPlayerTeam, kEnterpriseTeam, kTeamAllied, kGoalIgnore, 1.0, kTrue );
						fMission.mDisplayMessage( kPlayerCrippled_msg );
						fMission.mSetTimedInterval( kTimerLimitSet_4, 6 );
					}
					else
					{
						Ship->mSetShipCallbackEvents( kNotifyDamage );
					}
				}
			}
			else
			{
				if( Ship->mGetDamagePercent() > 80 )
				{
					fMission.mDisplayMessage( kEntCrippled_msg );
				}
				else if( Ship != NULL )
				{
					Ship->mSetShipCallbackEvents( kNotifyDamage );
				}
			}
			break;

		case kNotifyCommButtonPress:
			switch( fEventID )
			{
				case kButtonHail_text:
					if( Ship->mGetTeam() == kEnterpriseTeam )
					{
						fMission.mDisplayMessage( kEntHail_msg );
					}
					else if( Ship == OutpostTeam->mGetThisShip( Outpost4 )) 
					{
						fMission.mDisplayMessage( kHailOutpost4_msg, kFleetScriptMessage );
					}
					else if( Ship->mGetTeam() == kOutpostTeam )
					{
						fMission.mDisplayMessage( fMission.mRandomInt32( kHailOutpost1_msg, kHailOutpost3_msg ), kFleetScriptMessage );
					}
					else if( Ship->mGetTeam() == kAllyTeam )
					{
						fMission.mDisplayMessage( kAllyHail_msg, kFleetScriptMessage );
						Ship->mSetCommButton( 1, kButtonTransferParts_text );
						Ship->mSetCommButton( 2, kButtonTransferSupplies_text );
					}
					break;

				case kButtonTransferParts_text:
					if( Ship->mGetTacticalDistance( fPlayerShip ) > 5 )
					{
						fMission.mDisplayMessage( kAllyTooFar_msg, kFleetScriptMessage );
						Ship->mSetCommButton( 1, kButtonTransferParts_text );
					}
					else
					{
						fMission.mDisplayMessage( kAllyParts_msg, kFleetScriptMessage );
						fMission.mSetTimedInterval( kTimerTurnsSet_6, 1 );
						Ship->mClearCommButton( 2 );
					}
					break;

				case kButtonTransferSupplies_text:
					if( Ship->mGetTacticalDistance( fPlayerShip ) > 5 )
					{
						fMission.mDisplayMessage( kAllyTooFar_msg, kFleetScriptMessage );
						Ship->mSetCommButton( 2, kButtonTransferSupplies_text );
					}
					else
					{
						fMission.mDisplayMessage( kAllySupplies_msg, kFleetScriptMessage );
						fMission.mSetTimedInterval( kTimerTurnsSet_7, 1 );
						Ship->mClearCommButton( 1 );
					}
					break;
			}
			break;

		case kNotifyDisengaged:
			if( Ship->mGetTeam() == kPlayerTeam )
			{
				int32 MapX = 0;
				int32 MapY = 0;

				Ship->mGetGridPosition( &MapX, &MapY );

				if( MapX >= 19 )
				{
					GoodDisengage = true;

					if( DeadOutposts >= 4 )
					{
						return kPlayerWins;
					}
					else
					{
						return kPlayerLoses;
					}
				}
			}
			break;
	}
	return kGameInProgress;
}

//---------------------------------------------------------------------------------------------
// called when mission has ended but before debriefing and only if player is still alive
eVictoryLevels tScript::mCalcVictoryStatus (void)
{
	if(( PlayerDestroyed == true ) || ( GoodDisengage == false ))
	{
		return( kDevastatingDefeat );
	}

	if( EnterpriseCaptured == true )
	{
		return( kAstoundingVictory );
	}

	if(( EnterpriseDestroyed == true ) && ( DeadOutposts > 4 ))
	{
		return( kAstoundingVictory );
	}

	if(( EnterpriseDestroyed == true ) || ( DeadOutposts > 4 ))
	{
		return( kVictory );
	}

	// default
	return( kDraw );
}

//---------------------------------------------------------------------------------------------
// called at end of mission only if player is still alive
void tScript::mGetDebriefingList (eVictoryLevels VictoryLevel)
{
	if( PlayerDestroyed == true )
	{
		fMission.mAddToDebriefingList( kDBPlayerDied_msg );
		fMission.mAddToDebriefingList( kDBBad_msg );
		return;
	}

	if( GoodDisengage == false )
	{
		fMission.mAddToDebriefingList( kDBBadDisengage_msg );
		fMission.mAddToDebriefingList( kDBBad_msg );
		return;
	}
	else
	{
		fMission.mAddToDebriefingList( kDBGoodDisengage_msg );
	}

	fMission.mAddToDebriefingList( kDBKilled0_msg + DeadOutposts );

	if( CapturedOutpost == true )
	{
		fMission.mAddToDebriefingList( kDBCapturedOP_msg );
	}

	if( EnterpriseDestroyed == true )
	{
		fMission.mAddToDebriefingList( kDBDestroyedEnt_msg );
	}
	else if( EnterpriseCaptured == true )
	{
		fMission.mAddToDebriefingList( kDBCapturedEnt_msg );
	}

	if( DeadOutposts > 3 )
	{
		fMission.mAddToDebriefingList( kDBGood_msg );
	}
	else
	{
		fMission.mAddToDebriefingList( kDBBad_msg );
	}
}

//---------------------------------------------------------------------------------------------
// called at the end of every mission
int32 tScript::mGetTechnicalSuccessLevel (tDynaverseScriptInfo& info)
{
	return 0;
}

