// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
//
//  NOTE:	This is a complete production mission. For an explaination of the functions used refer to
//			ScriptInterface.h, ScriptStructs.h and Tst_Examples.cpp. Also see Gen_Template.cpp, 
//			His_Template.cpp, and Mul_Template.cpp for simple template missions.
//
// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<







#include "scriptinterface.h"

#include "MissionText.h"
#include "MissionMaps.h"

#define kRepairInnerDistance	10.0		// must be inside this distance to repair ship
#define kRepairOuterDistance	20.0		// must be outside this distance to reset inner distance
#define kDetonationDistance		10.0		// must be this close to a base to autodestroy puck and base

tString PlayersShipClassName;
iTruth RespawningLocalShip	= kFalse;

int32 kPuckToGet			= 1;
float Base1DamagePercent	= 0;
float Base2DamagePercent	= 0;
int32 Base1Hits				= 0;
int32 Base2Hits				= 0;
int32 fStarbaseToDefend;
int32 HumanTeams[tEntityMgr::kMaxNumberofPlayers];

//---------------------------------------------------------------------------------------------
// the following enum is used to identify all non-player teams in the script
// (use kPlayerTeam to access the current players team.)
enum eTeamTypes
{
	// the first 6 team types are reserved for player teams.
	kReserved = kTeam6,

	// add all non-player teams (enemy, allied, monsters, etc...) after this point.
	kTeam1StarBase,
	kTeam2StarBase,
	kTeamPuck,
};

//---------------------------------------------------------------------------------------------
// called by script interface - 
// should return whatever team is considered to be the primary enemy
int32 tScript::mGetEnemyTeamID (void)
{
	return kPlayerTeam;
}

//---------------------------------------------------------------------------------------------
// returns mission specs used by Dynaverse to build campaigns
void tScript::mScriptSpecs (tMissionSpec* Specs)
{
	Specs->fMissionType = kMultiplayerMission;
	Specs->fMissionTitle = Messages[kMissionTitle_msg].mCharStr();
	Specs->fSectorType = kAllSectors;
	Specs->fPlayerRace = kAllRaceMission;
	Specs->fEnemyRace = kAllRaceMission;
	Specs->fMinBPV = 0;
	Specs->fMaxBPV = kMaxBPV;
	Specs->fMinDate = 9999;
	Specs->fMaxDate = 9999;

	// multiplayer menu specific settings:
	//		max players (fWait)
	//		max ships (fNumShips)
	//		enable supply dock
	//		disable teams
	//		disable AI
	Specs->fNumShips = MissSpecOneShip;				// max number of ships per player
	Specs->fWait = 6;								// max number of players
	Specs->fEnableSupplyDock = kEnableSupplyDock | kDisableTeamC | kDisableTeamD | kDisableTeamE | kDisableTeamF | kDisableAI;
}

//---------------------------------------------------------------------------------------------
// called at the start of mission initialization - use to set prestige points 
// and override mission defaults such as:
//		allied race
//		enemy race
//		system name
// and also returns map index to use
int32 tScript::mInitializeStart (tDynaverseScriptInfo& info)
{
	// keep track of which teams the humans are on
	for (int32 i= 0; i< tEntityMgr::kMaxNumberofPlayers; i++)
	{
		HumanTeams[i] = -1;		// -1 == empty
		if (info.fMembersInGame[i] == kPlayerTypeHuman)
		{
			HumanTeams[i] = info.fMemberTeams[i];
		}
	}

	// determine which starbase the player is defending
	fStarbaseToDefend = (kPlayerTeam == kTeam1) ? kTeam1StarBase : kTeam2StarBase;

	// must setup the races and relations of the reserved teams in the script
	fMission.mSetMultiplayerTeams (info);

	// setup the races of the non-reserved teams in the script
	fMission.mSetTeamRace (kTeam1StarBase, kFederation);
	fMission.mSetTeamRace (kTeam2StarBase, kKlingon);
	fMission.mSetTeamRace (kTeamPuck, kFederation);

	// setup team relations and target goals of non-reserved teams
	fMission.mSetTeamRelations (kPlayerTeam, kTeam1StarBase, kTeamAllied, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kPlayerTeam, kTeam2StarBase, kTeamAllied, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kPlayerTeam, kTeamPuck, kTeamNeutral, kGoalIgnore, 1.0, kTrue);

	fMission.mSetTeamRelations (kTeam1StarBase, kTeam2StarBase, kTeamNeutral, kGoalIgnore, 1.0, kTrue);

	for (i = 0; i < kMaxMultiTeams; i++)
	{
		fMission.mSetTeamRelations (eMulti_TeamTypes(i), kTeam1StarBase, kTeamAllied, kGoalIgnore, 1.0, kTrue);
		fMission.mSetTeamRelations (eMulti_TeamTypes(i), kTeam2StarBase, kTeamAllied, kGoalIgnore, 1.0, kTrue);
		fMission.mSetTeamRelations (eMulti_TeamTypes(i), kTeamPuck, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
	}

	fMission.mSetTeamRelations (kTeam1StarBase, kTeamPuck, kTeamNeutral, kGoalIgnore, 1.0, kTrue);
	fMission.mSetTeamRelations (kTeam2StarBase, kTeamPuck, kTeamNeutral, kGoalIgnore, 1.0, kTrue);

	// pick a map
	return 1;
}

//---------------------------------------------------------------------------------------------
// return kTrue if players ship should be created for this mission
iTruth tScript::mUsePlayerShip (tShipInGame& Ship)
{
	// players on team 1 will use start positions G-L, team 2 uses U-Z
	if (fStarbaseToDefend == kTeam2StarBase)
	{
		Ship.fShipPosition += kStartPosition_U - kStartPosition_G;
	}
	return kTrue;
}

//---------------------------------------------------------------------------------------------
// called to create ships before or during the mission
// Mode is user supplied value (expect for 0, which is initial setup call)
void tScript::mCreateMissionShips (int32 Mode)
{
	// only the host needs to create the AI ships
	if (fThisScriptIsTheHost)
	{
		tTeamInfo* Team;

		// create team 1 star base
		Team = fMission.mGetTeamHandle (kTeam1StarBase);
		Team->mCreateShip (kMaxBPV, kClassStarBase, kClassStarBase, 100, kStartPosition_M);
		// create team 2 star base
		Team = fMission.mGetTeamHandle (kTeam2StarBase);
		Team->mCreateShip (kMaxBPV, kClassStarBase, kClassStarBase, 100, kStartPosition_T);
		//create puck
		Team = fMission.mGetTeamHandle (kTeamPuck);
		Team->mCreateShip ("N-PUCK", kStartPosition_Q);
	}
}

//---------------------------------------------------------------------------------------------
// called at the end of mission initialization - use to return briefing message
int32 tScript::mInitializeFinished (void)
{
	// return the index to the briefing message we want to use
	return (kBriefing_msg);
}

//---------------------------------------------------------------------------------------------
// called after all ships have been created and mission is ready to start.
// use to set callback events and change default ship configuration
void tScript::mStartMission (void)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);
	tShipInfo* Ship;

	// configure player ship(s)
	Ship = PlayerTeam->mGetFirstShip();
	Ship->mSetShipPlayerControlled();
	Ship->mTrackTacticalDistance(kRepairInnerDistance, 20, 16); // Set tracking distance to the sun
	PlayersShipClassName = Ship->mGetClassName();	// save for respawn

	// only the host needs to setup the AI ships and events
	if (fThisScriptIsTheHost)
	{
		fMission.mSetTimedInterval (kTimerLimitSet_2, 3);
	}

	fMission.mSetTimedInterval( kTimerLimitSet_4, 1 );
}

//---------------------------------------------------------------------------------------------
// called whenever a ship is created after a mission has already started
void tScript::mNewMissionShip (tShipInfo* Ship)
{
	if (RespawningLocalShip && Ship->mGetTeam() == kPlayerTeam)
	{
		// reset all systems of the ship - this will update the ship to its original state
		Ship->mDamageSystem (kDamageWeaponsSystemsAndShields, 0);
		Ship->mAccessSupplies (kResetAllItems, 0, 0);

		Ship->mSetShipPlayerControlled();
		RespawningLocalShip = kFalse;
	}
}

//---------------------------------------------------------------------------------------------
// called during mission when events occur
eGameStatus tScript::mHandleShipEvent (tShipInfo* Ship, iNotifyEvents Event)
{
	switch (Event)
	{
		// NOTE: only the host will get these timer messages
		case kNotifyAtTimedInterval:
			{
				switch (fEventID)
				{
					case kTimerLimitSet:
						{
							// this scripts player lost - create a new ship
							tTeamInfo* Team = fMission.mGetTeamHandle (kPlayerTeam);
							tShipInfo* Ship = Team->mGetFirstShip();
							RespawningLocalShip = kTrue;
							Team->mCreateShip (0, PlayersShipClassName.mCharStr(), kStartPosition_O);
							fMission.mDisplayMessage( kPlayerIsBack_msg, kNonCommScriptMessage );
						}
						break;

					case kTimerLimitSet_2:
						{
							tShipInfo* Base1 = fMission.mGetTeamHandle(kTeam1StarBase)->mGetFirstShip();
							tShipInfo* Base2 = fMission.mGetTeamHandle(kTeam2StarBase)->mGetFirstShip();
							tShipInfo* Puck = fMission.mGetTeamHandle(kTeamPuck)->mGetThisShip (kPuckToGet);

							// leave now if puck not within exploding distance of any base
							if (Puck->mGetTacticalDistance(Base1) > kDetonationDistance
								&& Puck->mGetTacticalDistance(Base2) > kDetonationDistance)
							{
								fMission.mSetTimedInterval (kTimerLimitSet_2, 3);
								break;
							}

							// explode puck
							Puck->mDamageSystem(kDamageInternalsWithPoints, 30000);
							Puck->mDamageSystem(kDamageExcessDamageSystem, 5000);

							// set timer to create a new puck in 5 seconds
							fMission.mSetTimedInterval (kTimerLimitSet_3, 5);
						}
						break;

					case kTimerLimitSet_3:
						{
							tTeamInfo* Team = fMission.mGetTeamHandle (kTeamPuck);
							Team->mCreateShip ("N-PUCK", kStartPosition_Q);
							kPuckToGet++;
							fMission.mSetTimedInterval (kTimerLimitSet_2, 3);
						}
						break;

					case kTimerLimitSet_4:
						if( fStarbaseToDefend == kTeam1StarBase )
						{
							fMission.mDisplayMessage( kYourBaseFed_msg, kNonCommScriptMessage );
						}
						else if( fStarbaseToDefend == kTeam2StarBase )
						{
							fMission.mDisplayMessage( kYourBaseKlingon_msg, kNonCommScriptMessage );
						}
						break;
				}
			}
			break;

		case kNotifyAtDeath:
			{
				if (Ship->mGetTeam() == kTeamPuck)
				{
					// apply damage to whichever starbase the puck is closest to
					float DamagePercent;

					tShipInfo* BaseToDamage = fMission.mGetTeamHandle(kTeam1StarBase)->mGetFirstShip();
					if (Ship->mGetTacticalDistance(BaseToDamage) <= kDetonationDistance)
					{
						Base1Hits++;
						Base1DamagePercent += 35.0;
						DamagePercent = Base1DamagePercent;
					}
					else
					{
						BaseToDamage = fMission.mGetTeamHandle(kTeam2StarBase)->mGetFirstShip();
						Base2Hits++;
						Base2DamagePercent += 35.0;
						DamagePercent = Base2DamagePercent;
					}

					// apply damage to base
					BaseToDamage->mDamageSystem(kDamageMaxSystems, DamagePercent);
					BaseToDamage->mDamageSystem(kDamageMaxShields, DamagePercent);
					if (DamagePercent >= 100.0)
					{
						BaseToDamage->mDamageSystem(kDamageInternalsWithPoints, 3000);
						BaseToDamage->mDamageSystem(kDamageExcessDamageSystem, 500);
					}
					else
					{
						if( fStarbaseToDefend == kTeam1StarBase )
						{
							fMission.mDisplayMessage( kHitYourBase0_msg + Base1Hits );
							fMission.mDisplayMessage( kHitTheirBase0_msg + Base2Hits );
						}
						else if( fStarbaseToDefend == kTeam2StarBase )
						{
							fMission.mDisplayMessage( kHitYourBase0_msg + Base2Hits );
							fMission.mDisplayMessage( kHitTheirBase0_msg + Base1Hits );
						}
					}
				}
				else if (Ship->mGetTeam() == kTeam1StarBase)
				{
					if (fStarbaseToDefend == kTeam1StarBase)
					{
						return kPlayerLoses;
					}
					return kPlayerWins;
				}
				else if (Ship->mGetTeam() == kTeam2StarBase)
				{
					if (fStarbaseToDefend == kTeam2StarBase)
					{
						return kPlayerLoses;
					}
					return kPlayerWins;
				}
				else if (Ship->mGetTeam() == kPlayerTeam)
				{
					fMission.mDisplayMessage (kPlayerIsDestroyed_msg, kNonCommScriptMessage);
					// set 8 second callback to restart game with a new ship
					fMission.mSetTimedInterval (kTimerLimitSet, 8);
				}
			}
			break;

		case kNotifyAtDistance:
			{
				if (Ship->mGetTrackingDistance() == kRepairInnerDistance)
				{
					Ship->mDamageSystem (kDamageWeaponsSystemsAndShields, 0);
					Ship->mAccessSupplies (kResetAllItems, 0, 0);
					fMission.mDisplayMessage (kSystemsRestored, kEngineerScriptMessage);
					Ship->mTrackTacticalDistance(kRepairOuterDistance + 10.0, 20, 16); 
				}
				else if (Ship->mGetTacticalDistance (20, 16) > kRepairOuterDistance)
				{
					Ship->mTrackTacticalDistance(kRepairInnerDistance, 20, 16); 
				}
				else
				{
					Ship->mTrackTacticalDistance(kRepairOuterDistance + 10.0, 20, 16); 
				}
			}
			break;

		case kNotifyHostChanged:
			fThisScriptIsTheHost = kTrue;
			fMission.mSetTimedInterval (kTimerLimitSet_2, 3);
			break;

		case kNotifyDisengaged:
			{
				fEventID = Ship->mGetTeam();

				if( fEventID == kTeamPuck )
				{
					tShipInfo* Puck = fMission.mGetTeamHandle(kTeamPuck)->mGetThisShip (kPuckToGet);
					Puck->mDamageSystem(kDamageInternalsWithPoints, 30000);
					Puck->mDamageSystem(kDamageExcessDamageSystem, 5000);
					// set timer to create a new puck in 5 seconds
					fMission.mSetTimedInterval (kTimerLimitSet_3, 5);
					return kGameInProgress;
				}

				// it is a player ship that went out of bounds....
				Ship->mDamageSystem( kDamageExcessDamageSystem, 100 );
				Ship->mDamageSystem( kDamageInternalsWithPoints, 300 );
			}
			break;

			// case falls thru
		case kNotifyPlayerLeaving:
			if (fEventID != kPlayerTeam)
			{
				HumanTeams[fEventID] = -1;
				int32 first_team = -1;
				for (int32 i = 0; i < tEntityMgr::kMaxNumberofPlayers; i++)
				{
					if (HumanTeams[i] != -1)
					{
						if (first_team == -1)
						{
							first_team = HumanTeams[i];
						}
						if (HumanTeams[i] != first_team)
						{
							return kGameInProgress;		// still have two teams in the game
						}
					}
				}
				// must be only one team left, and we're on it - we win!
				return kPlayerWins;
			}
			break;
	}
	return kGameInProgress;
}

//---------------------------------------------------------------------------------------------
// called when mission has ended but before debriefing and only if player is still alive
eVictoryLevels tScript::mCalcVictoryStatus (void)
{
	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (fStarbaseToDefend);
	tShipInfo* Ship = PlayerTeam->mGetFirstShip();

	if (Ship->mInPlay())
	{
		return kAstoundingVictory;
	}
	return kDevastatingDefeat;
}

//---------------------------------------------------------------------------------------------
// called at end of mission only if player is still alive
void tScript::mGetDebriefingList (eVictoryLevels VictoryLevel)
{
	if (VictoryLevel == kAstoundingVictory)
	{
		fMission.mAddToDebriefingList (kWinner);
	}
	else
	{
		fMission.mAddToDebriefingList (kLoser);
	}
}

//---------------------------------------------------------------------------------------------
// called at the end of every mission
int32 tScript::mGetTechnicalSuccessLevel (tDynaverseScriptInfo& info)
{
	return 0;
}
