// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
//
//  NOTE:	This file is for example only. It is not intended to be played as a regular mission.
//			See Gen_Template.cpp, His_Template.cpp, or Mul_Template.cpp for actual working missions.
//
// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<


#include "scriptinterface.h"				// must have this as the first include

#include "Tst_ExamplesText.h"					// must have these two includes
#include "Tst_ExamplesMaps.h"

//---------------------------------------------------------------------------------------------
// the following enum is used to identify all non-human player teams in the script
// (use kPlayerTeam to access the current human players team.)
enum eTeamTypes
{
	// NOTE: the first 6 team types are reserved for human player teams. DO NOT CHANGE THIS LINE!
	kReserved = kTeam6,

	// add all non-human player teams (enemy, allied, monsters, etc...) after this point.
	kEnemyTeam,
};

// -----------------------------------------------------------------------------------
// this area can be used to add extra enums, work buffers or whatever is needed

// load in functions to keep track of items player needs to deep scan
#include "DeepScanTracker.h"
tDeepScanTracker DeepScanTracker;

// waypoints for patrol are 1 based
#define MaxWaypoints	5
tMapPoint PatrolWaypoints[MaxWaypoints]=
{
	5,	10,
	7,	23,
	14,	23,
	3,	21,
	4,	6
};

// -----------------------------------------------------------------------------------

//---------------------------------------------------------------------------------------------
//			START OF SCRIPT ROUTINES
//---------------------------------------------------------------------------------------------
// returns mission specs used by Dynaverse to build campaign list
void tScript::mScriptSpecs (tMissionSpec* Specs)
{
	Specs->fMissionType = kCampaignMission;				// use values from eMissionTypes enum in missionspec.h
	Specs->fMissionTitle = Messages[kMissionTitle_msg].mCharStr();
	Specs->fSectorType = kAllSectors;					// use values from eMissSpecSect enum in missionspec.h
	Specs->fPlayerRace = kAllRaceMission;				// use values from eMissSpecRace enum in missionspec.h
	Specs->fEnemyRace = kAllRaceMission;				// use values from eMissSpecRace enum in missionspec.h
	Specs->fNumShips = MissSpecAllShips;				// use values from Bit masks in missionspec.h
	Specs->fMinBPV = 0;									// player must have at least this much BPV to play this mission
	Specs->fMaxBPV = kMaxBPV;							// player cannot have more than this much BPV to play this mission
	Specs->fMinDate = 0;								// player can play this mission starting on this year date (date starts at 0
														//   and advances 1 year at a time)
	Specs->fMaxDate = 9999;								// player will not be able to play this mission after this year date
	Specs->fWait = 0;									// has multiple uses depending on which fMissionType was specified above:
														// for kCampaignMission:
														//		specifies how many years Dynaverse will wait before selecting 
														//		this misison again. when fWait = kUniqueMission then this mission
														//		is played only once per campaign.
														// for kTutorialMission & kTechnicalMission:
														//		specifies this missions position in the academy menu. must be 0 - 5.
														// for kHistoricalMission & kMultiplayerMission:
														//		unused - should be 0
	Specs->fEnableSupplyDock = kEnableSupplyDock;		// enables/disables the Supply dock button in the briefing screen.
														// uses values from eMenuButtons enum in scriptstructs.h
}

//---------------------------------------------------------------------------------------------
// called at the start of mission initialization - use to set prestige points 
// and override mission defaults such as:
//		allied race
//		enemy race
//		system name
// and also returns map index to use
int32 tScript::mInitializeStart (tDynaverseScriptInfo& info)
{
	// ----- functions to use in this section ----- 

	fMission.mSetPrestigePoints (100, 80, 0, -10, -50);

	fMission.mSetTeamRace (kPlayerTeam, info.fPlayerRace);
	fMission.mSetTeamRace (kEnemyTeam, kKlingon);

	fMission.mSetTeamRelations (kPlayerTeam, kEnemyTeam, kTeamAtWar, kGoalDestroy, kTrue);

	// multiplayer and skirmish setup calls
	//fMission.mSetMultiplayerTeams (info);
	//fMission.mSetSkirmishTeams (info);

	// ------ Not all tMissionInfo functions are usable in this section ------
	// ------ Not all tTeamInfofunctions are usable in this section ------
	// ------ DO NOT use any tSshipInfo functions in this section -------

	// pick a map
	return 1;
}

//---------------------------------------------------------------------------------------------
// return kTrue if players ship should be created for this mission.
// can also change the default starting position of the ship (Ship.fShipPosition).
// all players ships start at kStartPosition_G and increament from there.
// (you are not able to specify an X/Y grid location)
iTruth tScript::mUsePlayerShip (tShipInGame& Ship)
{
	return kTrue;
}

//---------------------------------------------------------------------------------------------
// called to create ships before the mission starts with Mode = 0.
// also can be called after the mission starts (Mode can be any user supplied value (expect 0)
void tScript::mCreateMissionShips (int32 Mode)
{
	// ----- functions to use in this section ----- 

	tTeamInfo* PlayerTeam = fMission.mGetTeamHandle (kPlayerTeam);

	PlayerTeam->mGetTeam();
	PlayerTeam->mGetRace();
	PlayerTeam->mGetShipsInPlayCount();
	PlayerTeam->mGetMultiplayerTeam();

	PlayerTeam->mCreateShip ("F-FF", kStartPosition_G); 
	PlayerTeam->mCreateShip (kMaxBPV, kClassFrigate, 50, kStartPosition_G);
	PlayerTeam->mCreateShip (kMaxBPV, kClassFrigate, kClassHeavyCruiser, 50, kStartPosition_G);
	PlayerTeam->mCreateFleet (3, kMaxBPV, kClassFrigate, kClassHeavyCruiser, 50, kStartPosition_G);

	// ------ DO NOT use the following tMissionInfo functions in this section -------
	//
	// mSetTimedInterval (eTimerIntervalTypes Type, int32 Interval);
	// mSetGoalIndicator (tShipInfo* TargetShip);
	// mSetGoalIndicator (iTruth IndicatorOn, int32 GridX, int32 GridY);		// Grid X/Y are 1 based
	// mGetShipTarget (tShipInfo* Ship);
	// mDisplayMessage (int32 MsgID, eScriptMessageTypes Type = kMissionScriptMessage, tShipInfo* Ship = NULL);
	// mFindShipID (int32 ShipID);
	// mStartFlashingButton (const char* ButtonID);
	// mStopFlashingButton (const char* ButtonID);

	// ------ DO NOT use the following tTeamInfo functions in this section -------
	//
	// mClearCommands (void);
	// mDisengageShips (eDisengageMethods Method, eAIPriority Priority);
	// mDefendEntity (tShipInfo* Entity, float ThreatRadius, float FollowRadius, eAIPriority Priority);

	// ------ DO NOT use any tShipInfo functions in this section -------

}

//---------------------------------------------------------------------------------------------
// called at the end of mission initialization - use to return index to briefing message
int32 tScript::mInitializeFinished (void)
{
	// return the index to the briefing message we want to use
	return (kBriefing_msg);
}

//---------------------------------------------------------------------------------------------
// called after all ships have been created and mission is ready to start.
// use to set callback events and change default ship configuration
void tScript::mStartMission (void)
{
	tTeamInfo* EnemyTeam = fMission.mGetTeamHandle (kEnemyTeam);
	tShipInfo* Ship;

	for (Ship = EnemyTeam->mGetFirstShip (); Ship != NULL; Ship = EnemyTeam->mGetNextShip ())
	{
		Ship->mSetCommButton (1, kButton1_Text);
		Ship->mSetCommButton (2, kButton2_Text);
	}

	// setup to test deep scan every second - must be called before any ships are added
	DeepScanTracker.mSetTimer (*this, kTimerLimitSet_10, 1);	// will use kTimerLimitSet_10 as polling timer for Deep Scan
	// deep scan enemy from within distance 20 for 4 turns
	DeepScanTracker.mSetTarget (EnemyTeam->mGetThisShip(1), 20.0, 4);

	fMission.mSetTimedInterval (kTimerTurnsSet, 2);			// set timer event to occur in 2 turns, based on game speed
	fMission.mSetTimedInterval (kTimerLimitSet, 4);			// set timer event to occur in 4 seconds, regardless of game speed


	// setup a patrol around 5 waypoints
	EnemyTeam->mGetFirstShip()->mPatrol (10.0, MaxWaypoints, &PatrolWaypoints[0], kAbsolutePriority);
}

//---------------------------------------------------------------------------------------------
// should return whatever team is considered to be the primary enemy.
// NOTE: should only return values found in eTeamTypes enum above
int32 tScript::mGetEnemyTeamID (void)
{
	return kEnemyTeam;
}

//---------------------------------------------------------------------------------------------
// called whenever a ship is created after a mission has already started
void tScript::mNewMissionShip (tShipInfo* Ship)
{
}

//---------------------------------------------------------------------------------------------
// called during the mission when events occur
eGameStatus tScript::mHandleShipEvent (tShipInfo* Ship, iNotifyEvents Event)
{
	switch (Event)
	{
		case kNotifyAtDeath:
			// Ship = ship that died.
			break;
		case kNotifyOnCapture:
			// Ship = ship that was captured. (Ships team is still old team at this point)
			// fEventID = ID of ship that captured it.
			break;
		case kNotifyDisengaged:
			// Ship = ship that disengaged.
			break;
		case kNotifyAtDistance:
			// Ship = ship that has just crossed into or out off range.
			// fEventID = inside/outside flag. 1 = ship has just crossed into range, 0 = ships crossed out of range
			break;
		case kNotifyAtDestination:
			// Ship = ship that has reached its destination.
			break;
		case kNotifyAtTimedInterval:
			// fEventID = timed interval that has expired. see eTimerIntervalTypes enum in scriptstructs.h
			switch (fEventID)
			{
				case kTimerLimitSet:
					fMission.mSetTimedInterval (kTimerLimitSet, 4);	 // reset timer event to occur in 4 seconds
					break;

				case kTimerLimitSet_10:
					{
						eDeepScanFlags flag;
						tShipInfo* Object = DeepScanTracker.mTestForDeepScan (&flag);
						switch (flag)
						{
							case kOutOfRange_ScanOff:
								// player is not within any objects range and deep scan is off (Object == NULL)
								break;
							case kOutOfRange_ScanOn:
								// player is not within any objects range and deep scan is on (Object == NULL)
								break;
							case kEnteringRange_ScanOff:
								// player has just entered range and scan is off (Object points to entity)
								// notify player to turn scan on
								break;
							case kEnteringRange_ScanOn_TimerStarted:
								// player has just entered range and scan is on (Object points to entity)
								// notify player that timer has begun
								break;
							case kInRange_ScanOff_TimerStopped:
								// player is in range and scan is off (Object points to entity)
								// notify player that timer has been turned off
								break;
							case kInRange_ScanOff:
								// player is still in range and scan is still off (Object points to entity)
								break;
							case kInRange_ScanOn_TimerStarted:
								// player is in range and scan is on (Object points to entity)
								// notify player that timer has begun
								break;
							case kInRange_ScanOn:
								// player is still in range and scan is still on (Object points to entity)
								break;
							case kLeavingRange_ScanOn_TimerStopped:
								// player has just left the range and scan is on (Object points to entity)
								// notify player that timer is now off
								break;
							case kLeavingRange_ScanOff:
								// player has just left the range and scan is off (Object points to entity)
								break;
						}
					}
					break;

				case kTimerCountdownSet:
					{
						eDeepScanFlags flag;
						tShipInfo* Object = DeepScanTracker.mTestForDeepScan (&flag);
						if (flag == kInRange_ScanOn)
						{
							// player has successfully scanned the object
							DeepScanTracker.mRemoveTarget (Object);
						}
					}
					break;
			}
			break;
		case kNotifyCommButtonPress:
			// Ship = target ship player is trying to communicate with.
			// fEventID =  message ID assigned to button.
			switch (fEventID)
			{
				case kButton1_Text:
					// the third value of kTrue will force the players current ship to select this ship as
					// a target and immediately display the Comm panel (if in view)
					Ship->mSetCommButton (1, kButton1_Text2, kTrue);
					break;

				case kButton2_Text:
					Ship->mSetCommButton (2, kButton2_Text2);
					break;

				case kButton2_Text2:
					Ship->mSetCommButton (1, kButton1_Text);
					Ship->mSetCommButton (2, kButton2_Text);
					break;
			}
			break;
		case kNotifyDamage:
			// Ship = ship that is taking damage.
			// fEventID = ID of ship that caused the damage.
			// fEventItem1 = type of damage done to ship - see eDamageToShip enum in scriptstructs.h
			// when fEventItem1 = kDamageWeaponByMarines:
			//		fEventItem2 == which weapon was hit: kDamageHeavyWeapon1 - kDamageHeavyWeapon25
			//		fEventItem3 == weapons current health; 0 == weapon destroyed
			// when fEventItem1 = kDamageSystemByMarines:
			//		fEventItem2 == which system was hit: kDamageSensorSystem, 
			//											 kDamageTransporterSystem,
			//											 kDamageTractorSystem
			//											 kDamageShuttleBaySystem
			//		fEventItem3 == systems current health; 0 == system destroyed
			break;
		case kNotifyDeepScan:
			// Ship = ship that has changed deep scan state
			// fIsDeepScanOn = state of deep scan. 0 = off, 1 = on.
			break;
		case kNotifyAlertStatusChange:
			// Ship = ship that has changed alert status.
			// fEventID = alert status level. see eAlertLevels enum in scriptstructs.h
			break;
		case kNotifyProbeLaunch:
			// Ship = ship that launched the probe
			// fEventID = ID of probe target.
			break;
		case kNotifyBeginOrbit:
			// Ship = ship that has begun orbit
			// fEventID = ID of object that ship is orbiting.
			break;
		case kNotifyEndOrbit:
			// Ship = ship that has ended orbit
			// fEventID = ID of object that ship was orbiting.
			break;
		case kNotifyBeginTractor:
			// Ship = ship that has begin tractoring
			// fEventID = ID of object that ship is tractoring
			break;
		case kNotifyEndTractor:
			// Ship = ship that has ended tractoring
			// fEventID = ID of object that ship was tractoring
			break;
		case kNotifyFlashingButtonPressed:
			// notification that the button set up to flash has been pressed
			break;
		case kNotifyTransportedItemTo:
			// Ship = ship that transported item
			// fEventID = ID of ship that item was transported to.
			// fEventItem1 = item that was transported. see eScriptTransportItems enum in scriptstructs.h
			break;
		case kNotifyTransportedItemFrom:
			// Ship = ship that received transported item
			// fEventID = ID of ship that sent transported item.
			// fEventItem1 = item that was transported. see eScriptTransportItems enum in scriptstructs.h
			break;
		case kNotifyChangingShip:
			// Ship = ship that player is leaving
			// fEventID = ID of ship that player is changing to.
			// (fPlayerShip has already been set to new ship)
			break;
		case kNotifySpecialTransport:
			// Ship = ship that transported marines
			// fEventID = type of marines transported, kBoardingParty or kHitAndRun.
			break;
		case kNotifyPlayerLeaving:
			// fEventID = team number of person leaving
			break;
		case kNotifyHostChanged:
			// this script has been selected as the new host in a multiplayer game
			break;
	}
	return fMission.mGetGameStatus();
}

//---------------------------------------------------------------------------------------------
// called when mission has ended but before debriefing and only if player is still alive
eVictoryLevels tScript::mCalcVictoryStatus (void)
{
	return (fMission.mGetVictoryLevel (0, 0));
}

//---------------------------------------------------------------------------------------------
// called at end of mission only if player is still alive
void tScript::mGetDebriefingList (eVictoryLevels VictoryLevel)
{
}

//---------------------------------------------------------------------------------------------
// called at the end of every mission
int32 tScript::mGetTechnicalSuccessLevel (tDynaverseScriptInfo& info)
{
	return 0;
}

