//	<Name>	scriptstructs.h
//
//     <Legal>     Copyright (c) 1997-1999 Quicksilver Software, Inc.
//     Permission is granted to use this file solely in accordance with the terms and
//     conditions in the SFC Mission API Software Use Limitations and Limited License.
//     All other rights reserved.
//
//     <Module>     Star Trek: Starfleet Command Mission API
//

#ifndef scriptstructs_h
#define scriptstructs_h

#include "tstring.h"
#include "eracename.h"
#include "raceenums.h"
#include "eshiptypes.h"
#include "esectorname.h"
#include "missionspec.h"
#include "entitymgr.h"
#include "team.h"

const int ScriptVersion = 57;

const int32 kMaxBPV				=	32767;
const int32 kMaxCPUTeams		=	5;
const int32 kMaxScriptValues	=	16;

	// events game notifies script of
typedef int32 iNotifyEvents;
enum
{
	kNotifyOff						=	0x0,

	// these notifies are single events only. must be reset after each callback to continue notify
	kNotifyAtDeath					=	0x00000001,
	kNotifyOnCapture				=	0x00000002,
	kNotifyDisengaged				=	0x00000004,
	kNotifyAtDistance				=	0x00000008,
	kNotifyAtDestination			=	0x00000010,
	kNotifyAtTimedInterval			=	0x00000020,
	kNotifyCommButtonPress			=	0x00000040,
	kNotifyDamage					=	0x00000080,			// sent along with an eDamageToShip value
	kNotifyDeepScan					=	0x00000100,
	kNotifyAlertStatusChange		=	0x00000200,
	kNotifyProbeLaunch				=	0x00000400,
	kNotifyBeginOrbit				=	0x00000800,
	kNotifyEndOrbit					=	0x00001000,
	kNotifyBeginTractor				=	0x00002000,
	kNotifyEndTractor				=	0x00004000,
	kNotifyFlashingButtonPressed	=	0x00008000,
	kNotifyTransportedItemTo		=	0x00010000,
	kNotifyTransportedItemFrom		=	0x00020000,
	kNotifyChangingShip				=	0x00040000,
	kNotifySpecialTransport			=	0x00080000,
	kNotifyPlayerLeaving			=	0x00100000,
	kNotifyHostChanged				=	0x00200000,
};
// defaults set automatically for every ship created
#define kNotifyDefaultEvents (kNotifyAtDeath + kNotifyOnCapture + kNotifyDisengaged + kNotifyChangingShip)

	// events script notifies game of
typedef int32 iCallbackEvents;
enum
{
	kGetTacticalDistance		=	0x00000001,
	kDamageShipSystem			=	0x00000002,
	kAddToCareerTotal			=	0x00000004,
	kSetCommButton				=	0x00000008,
	kPlaySound					=	0x00000010,
	kGetPlayerRank				=	0x00000020,
	kGetAlertStatus				=	0x00000040,
	kCreateMapEntry				=	0x00000080,
	kGetMapEntryPosition		=	0x00000100,
	kGetShipPosition			=	0x00000200,
	kSetTimedInterval			=	0x00000400,
	kTrackDistanceFromTarget	=	0x00000800,
	kGetINIFileEntry			=	0x00001000,
	kIssueAICommand				=	0x00002000,
	kSetFinalDestination		=	0x00004000,
	kSetPlayerControlledShip	=	0x00008000,
	kSetNewThrust				=	0x00010000,
	kSetNewEvents				=	0x00020000,
	kGetRaceRelationship		=	0x00040000,
	kSetTeamRelationship		=	0x00080000,
	kGetShipClassName			=	0x00100000,
	kGetShipTargetID			=	0x00200000,
	kSetObjectName				=	0x00400000,
	kGetShipDamagePercent		=	0x00800000,
	kScheduleMission			=	0x01000000,
	kStartFlashingButton		=	0x02000000,
	kGetSetOfficerRank			=	0x04000000,
	kAccessShipSupply			=	0x08000000,
	kMakeShipTargetable			=	0x10000000,
	kDynaverseAdjustments		=	0x20000000,
};

enum eDamageToShip
{
	kDamageByWeaponOrDebris,
	kDamageByProbe,
	kDamageByMine,
	kDamageWeaponByMarines,
	kDamageSystemByMarines,
};

enum eDamageSystems
{
	kDamageHeavyWeapon1,
	kDamageHeavyWeapon2,
	kDamageHeavyWeapon3,
	kDamageHeavyWeapon4,
	kDamageHeavyWeapon5,
	kDamageHeavyWeapon6,
	kDamageHeavyWeapon7,
	kDamageHeavyWeapon8,
	kDamageHeavyWeapon9,
	kDamageHeavyWeapon10,
	kDamageWeapon11,
	kDamageWeapon12,
	kDamageWeapon13,
	kDamageWeapon14,
	kDamageWeapon15,
	kDamageWeapon16,
	kDamageWeapon17,
	kDamageWeapon18,
	kDamageWeapon19,
	kDamageWeapon20,
	kDamageWeapon21,
	kDamageWeapon22,
	kDamageWeapon23,
	kDamageWeapon24,
	kDamageWeapon25,

	kDamageMaxWeapons,

	// the following must match the sequence of eSystemHardPoint in sysbaseenums.h
 	kDamageBridgeSystem,
	kDamageSensorSystem,
		kDamageHullGroupHardPoint,			// Placeholder - DO NOT USE!
	kDamageExcessDamageSystem,
	kDamageTransporterSystem,
	kDamageTractorSystem,
	kDamageShuttleBaySystem,
		kDamageFighterBayHardPoint,			// Placeholder - DO NOT USE!
		kDamageEngineGroupHardPoint,		// Placeholder - DO NOT USE!
	kDamageBatterySystem,
	kDamageLabSystem,
	kDamageBarracksSystem,
	kDamageCargoSystem,
		kDamageShieldGeneratorHardPoint,	// Placeholder - DO NOT USE!
	kDamageArmorSystem,
	kDamageCloakSystem,
	kDamageDamageControlGroupSystem,
		kDamageOfficerGroupHardPoint,		// Placeholder - DO NOT USE!
	kDamageProbeSystem,
	kDamageLeftWarpEngine,
	kDamageCenterWarpEngine,
	kDamageRightWarpEngine,
	kDamageImpulseEngine,
	kDamageAPRSystem,
	kDamageFlagBridgeSystem,
	kDamageEmergencyBridgeSystem,
	kDamageAuxilliaryControlSystem,
	kDamageScannerSystem,
	kDamageDamageControlSystem,
	kDamageInternalRepairSystem,
	kDamageForwardHullSystem,
	kDamageCenterHullSystem,
	kDamageAftHullSystem,
	kDamageTractorGroupSystem,

	kDamageMaxSystems,

	kDamageWeaponsAndSystems,

	kDamageShield_1_Fore,
	kDamageShield_2_BottomFore,
	kDamageShield_3_BottomAft,
	kDamageShield_4_Aft,
	kDamageShield_5_TopAft,
	kDamageShield_6_TopFore,

	kDamageMaxShields,

	kDamageWeaponsSystemsAndShields,

	kDamageInternalsWithPoints,				// randomly damage internal systems - 5000 points will explode ship
	kDamageInternalsWithPointsSilently,		// same as above but makes no sound
};

enum eShipSupplyAccess
{
	kGetSupplyCount,				// uses values from eShipSupplies
	kSetSupplyCount,				// uses values from eShipSupplies
	kGetHeavyWeaponCount,			// uses values from eDamageSystems
	kSetHeavyWeaponCount,			// uses values from eDamageSystems
	kGetTransporterItemCount,		// uses values from eScriptTransportItems
	kSetTransporterItemCount,		// uses values from eScriptTransportItems
	kGetFighterItem,				// uses values from eScriptFighters
	kSetFighterItem,				// uses values from eScriptFighters

	kResetSupplyCounts,
	kResetHeavyWeapons,
	kResetTransporters,
	kResetFighters,					// only resets fighter count, not fighter type
	kResetAllItems,
};

enum eShipSupplies
{
	kSupplyMissileType,
	kSupplyMissileSpeed,
	kSupplyNumberOfReloads,
	kSupplyNumberLoaded,
	kSupplyShuttles,
	kSupplySuicides,
	kSupplyWildWeasel,
	kSupplyScatterPack,
	kSupplySpareParts,
	kSupplyMarines,
	kSupplyMines,
};

enum eScriptFighters
{
	kFighterBay1Count,
	kFighterBay2Count,
	kFighterBay3Count,
	kFighterBay4Count,

	kFighterBay1GroupType,
	kFighterBay2GroupType,
	kFighterBay3GroupType,
	kFighterBay4GroupType,

	kFighterPatrolCount,			// used only by kGetFighterItem
	kFighterInterceptCount,
	kFighterHeavyCount,
	kFighterAssaultCount,
};

//ZDNOTE I would like to move this into an AI namespace.. will worry about that latter
enum eAICommands
{
	kAITargetObject,
	kAIClearCommands,
	kAIDisengage,
	kAIPatrol,
	kAIDefendObject,
	kAIMoveObject,
	kAIFollowObject,
	kAIFireProbe,
	kAIChangeOfficerState,	// uses values from eAIOfficers below
	kAICloakShip,
};

// must match eOfficer enum in aiCaptian.h
enum eAIOfficers
{
	kShieldReinforcementOfficer = 0,
	kECMOfficer,
	kECCMOfficer,
	kTractorOfficer,
	kRepairOfficer,
	kSeekingWeaponDefenseOfficer,
	kHitAndRunOfficer,
	kTBombOfficer,
	kHETOfficer,
	kCloakOfficer,
	kFighterOfficer,
	kPlanetKillerOfficer,
};


enum eAIPriority
{ 
	kNeverPriority = 0,
	kVeryLowPriority = 1,
	kLowPriority= 3,
	kMediumPriority = 5,
	kHighPriority = 7,
	kVeryHighPriority = 9,
	kAbsolutePriority = 10,
};

// must match one-to-one with enum eTransporterMission in transporter.h
enum eScriptTransportMission
{
	kTransNoMission = 0,
	kTransAwayTeam,
	kTransBoardingParty,
	kTransHitAndRun,
	kTransSendBomb,
	kTransRetrieve,
	kTransSendItem,
	kTransMoveCaptain
};
// must match one-to-one with enum eTransporterItem in transporter.h
enum eScriptTransportItems
{
	kTransNothing = 0,			//kNothing
	kTransMarine,
	kTransGravTank,
	kTransCaptain,
	kTransGornEgg,
	kTransCrew,
	kTransHeavyWeaponPlatform,
	kTransSpy,
	kTransSparePart,
	kTransScientists,
	kTransMedicalSupplies,
	kTransTribbles,
	kTransRoyalGornEgg,
	kTransDiplomat,
	kTransCommandos,
	kTransRomulanAle,
	kTransGenericPiratedCargo,
	kTransDilithiumCrystals,
    kTransBlackBox,
    kTransDeathPlague,
    kTransHarryMudd,
    kTransInfiltrator,
    kTransIonicProjector,
    kTransLifePodDude,
    kTransMedicineJar,
    kTransNovaMine,
    kTransPsionicDisruptor,
    kTransWeaponSchematic,
	kTransLastGNRAlbum
};

enum eAlertLevels
{
	kAlertGreen,
	kAlertYellow,
	kAlertRed,
};

enum eTimerIntervalTypes
{
	kTimerEndGamePause,			// use this to adjust the amount of seconds to pause before leaving the tactical game
	kTimerCountdownSet,			// use this to display a countdown timer on the tactical screen
	kTimerCountdownAdjust,		// use this to modify the countdown timer

	kTimerTurnsSet,				// use thses to setup a timer callback using game turn time
	kTimerTurnsSet_2,
	kTimerTurnsSet_3,
	kTimerTurnsSet_4,
	kTimerTurnsSet_5,
	kTimerTurnsSet_6,
	kTimerTurnsSet_7,
	kTimerTurnsSet_8,
	kTimerTurnsSet_9,
	kTimerTurnsSet_10,

	kTimerLimitSet,				// use these to setup a timer callback using seconds
	kTimerLimitSet_2,
	kTimerLimitSet_3,
	kTimerLimitSet_4,
	kTimerLimitSet_5,
	kTimerLimitSet_6,
	kTimerLimitSet_7,
	kTimerLimitSet_8,
	kTimerLimitSet_9,
	kTimerLimitSet_10,
};

enum eOfficerTypes
{
	kWeaponsOfficer,
	kEngineeringOfficer,
	kScienceOfficer,
	kCommOfficer,
	kHelmOfficer,
	kSecurityOfficer,
	kCaptainOfficer,
	kMaxOfficers
};

enum eOfficerRank
{
	kUnknownRank = 0,
	kRookie = 1,
	kJunior,
	kSenior,
	kVeteran,
	kLegendary,
	kMaxOfficerRank
};

enum eScriptMessageTypes
{ 
	kEngineerScriptMessage,
	kMissionScriptMessage,
	kScienceScriptMessage,
	kFleetScriptMessage,
	kNonCommScriptMessage,
};

enum eGameStatus
{
	kGameInProgress		=	0x00000000,			// must be first

	// player wins mission events
	kEnemyCaptured		=	0x00000001,
	kEnemyDestroyed		=	0x00000002,
	kEnemyDisengaged	=	0x00000004,
	kPlayerWins			=	0x00000008,

	// player loses mission events
	kPlayerDisengaged	=	0x00000100,
	kAllyDestroyed		=	0x00000200,
	kAllyCaptured		=	0x00000400,
	kAllyDisengaged		=	0x00000800,
	kPlayerLoses		=	0x00001000,

	// player kicked from mission events
	kMissionFailed		=	0x00010000,
	kPlayerDestroyed	=	0x00020000,
	kPlayerCaptured		=	0x00040000,

	// extra flags to tack on to status above
	kExpellFromElite	=	0x40000000,
	kCampaignCompleted	=	0x80000000,
};


// the following values are used in mSetTeamRelations calls with values from eTeamRelationship enum.
// different combinations have certain affects -
// when goal is kGoalIgnore:
//		for kTeamAllied, kTeamFriendly, kTeamNeutral - goal has no affect
//		for kTeamHostile	- will return fire when fired upon and advance to At War/Destroy status
//		for kTeamAtWar		- will fire on enemy if within close proximity.
// when goal is kGoalCapture:
//		for kTeamAllied, kTeamFriendly, kTeamNeutral - goal has no affect
//		for kTeamHostile	- when fired upon it will advance to At War and try to capture
//		for kTeamAtWar		- will move in to capture
// when goal is kGoalDestroy:
//		for kTeamAllied, kTeamFriendly, kTeamNeutral - goal has no affect
//		for kTeamHostile	- when fired upon it will advance to At War and try to destroy
//		kTeamAtWar			- will move in to destroy
enum eTargetGoals
{
	kGoalIgnore,
	kGoalCripple,	// - not implemented
	kGoalCapture,
	kGoalDestroy,
};

// if adding or deleting from this list be sure to change dynaverse.txt also
// and rebuild a new strings.txt file
enum eClassTypes
{
	kNoClassType = -1,
	kClassShuttle = 0,				// smallest hull size
	kClassFreighter,
	kClassFrigate,
	kClassDestroyer,
	kClassWarDestroyer,
	kClassLightCruiser,
	kClassNewHeavyCruiser,
	kClassHeavyCruiser,
	kClassHeavyBattlecruiser,
	kClassDreadnought,
	kClassBattleship,				// largest hull size
	kClassBaseStation,
	kClassBattleStation,
	kClassStarBase,
	kClassMonster,
	kClassPlanets,
	kClassSpecial,
	kMaxClasses
};

// if adding or deleting from this list be sure to change dynaverse.txt also
// and rebuild a new strings.txt file
enum eSystems
{
	kFirstSystem,
	kSecondSystem,
	kThirdSystem,
	kFourthSystem,
	kFifthSystem,
	kSixthSystem,
	kSeventhSystem,
	kEighthSystem,
	kNinthSystem,
	kTenthSystem,
	kMaxSystems
};

enum eMenuButtons
{
	kEnableSupplyDock			=	0x00000001,
	kDisableCustomSkirmish		=	0x00000002,
	kDisableAI					=	0x00000004,
	kDisableTeamA				=	0x00000008,
	kDisableTeamB				=	0x00000010,
	kDisableTeamC				=	0x00000020,
	kDisableTeamD				=	0x00000040,
	kDisableTeamE				=	0x00000080,
	kDisableTeamF				=	0x00000100,

	kDisableAllTeams			=	(kDisableTeamA | kDisableTeamB | kDisableTeamC | kDisableTeamD | kDisableTeamE | kDisableTeamF),
	kDisableAll					=	(kDisableCustomSkirmish | kDisableAI | kDisableAllTeams),
};

//-------------------------------------------------------
// available victory levels
enum eVictoryLevels
{
	kAstoundingVictory,
	kVictory,
	kDraw,
	kDefeat,
	kDevastatingDefeat,
	kMaxVictoryLevels,
	kBonusPrestige,
};

// must match eTableType in raceenum.h
enum eDynaverseTableType
{
	kAdjustEconomy,
	kAdjustGovStability,
	kAdjustFirstPolicy,
	kAdjustSecondPolicy,
	kAdjustPiratePolicy,
	kAdjustForeignRelation,
	kAdjustWarProgression,
};

//-------------------------------------------------------
// Career event types
enum eCareerEvents
{
	kDestroyed,
	kCaptured,
	kCrippled,
	kDefended,
	kMaxEvents
};

//-------------------------------------------------------
// used to place ships on the map
enum eStartPositions
{
	kStartPosition_Specified,		// must also include StartX and StartY in same call (EndX, EndY are optional)

	kStartPosition_G,
	kStartPosition_H,
	kStartPosition_I,
	kStartPosition_J,
	kStartPosition_K,
	kStartPosition_L,
	kStartPosition_M,
	kStartPosition_N,
	kStartPosition_O,

	kStartPosition_P,
	kStartPosition_Q,
	kStartPosition_R,

	kStartPosition_S,
	kStartPosition_T,
	kStartPosition_U,
	kStartPosition_V,
	kStartPosition_W,

	kStartPosition_X,
	kStartPosition_Y,
	kStartPosition_Z,

	kGroupAround	=	0x80000000	// OR this value to one of the above to create a fleet around the position
};

struct tShipSystemsInfo
{
public:
	iNotifyEvents	fAction;
	int32			fShipEntityID;
	eTeamID			fTeam;

	int32			fGridX;
	int32			fGridY;

	// work variables used to pass data between script and game engine
	int32			fInt32[5];
	float			fFloat[5];
	tString			fText;
	const char*		fTextPtr;
};

struct tShipInGame
{
public:
	const char*		fClassName;
	const char*		fCustomName;
	eClassTypes		fClassType;
	int32			fShipBPV;
	int32			fShipPosition;
	int32			fStartX;
	int32			fStartY;
	int32			fEndX;
	int32			fEndY;
	eTeamID			fTeam;
	int32			fScriptCallbackID;
	int32			fPlayerShipIndex;
	iTruth			fCreateShadowShip;
};

struct tCreateShipInfo
{
public:
	tShipInGame		fShipData;

	eClassTypes		fClassTypeLast;		// specifies last ship type for fleet creation
	eRaceName		fShipRace;
	int32			fVariant;
	int32			fTotalBPV;
	eClassTypes		fSelectedClassType;
};

enum eMultiplayerMembers
{
	kPlayerTypeNoOne		=	0,
	kPlayerTypeAI			=	1,
	kPlayerTypeHuman		=	2,
	kPlayerTypeWatch		=	3
};

struct tDynaverseScriptInfo
{
public:
	//----------------------------------------
	// members filled in by Dynaverse
	//----------------------------------------
	struct tMissionDate
	{
		int32u		fDay;
		int32u		fYear;
	};


	tString				fMissionName;
	eSectorName			fMissionLocation;
	tMissionDate		fMissionDate;
	eMissionTypes		fMissionType;
	int32				fCampaignYear;

	eTeamID				fPlayerTeam;
	eRaceName			fPlayerRace;
	eRaceName			fAlliedRaces[kNumberOfRaces + 1];
	eRaceName			fEnemyRaces[kNumberOfRaces + 1];
	eRaceName			fPreviousAlliedRace;
	eRaceName			fPreviousEnemyRace;
	eVictoryLevels		fPreviousVictoryLevel;
	eSystems			fPreviousSystem;

	int32				fPlayerFleetSize;
	tShipInGame			fPlayerFleet[3];

	eSystems			fSystemNames[kMaxSystems + 1];

	int32				fPreferedMap;

	// multiplayer/skirmish specific
	eMultiplayerMembers	fMembersInGame[tEntityMgr::kMaxNumberofPlayers];
	eRaceName			fMemberRaces[tEntityMgr::kMaxNumberofPlayers];
	eTeamID				fMemberTeams[tEntityMgr::kMaxNumberofPlayers];
	iTruth				fMemberIsLocal[tEntityMgr::kMaxNumberofPlayers];
	int32				fMemberShipCounts[tEntityMgr::kMaxNumberofPlayers];
	int32				fMemberBPVLimits[tEntityMgr::kMaxNumberofPlayers];
	int32				fMemberBPVTeamLimits[tEntityMgr::kMaxNumberofPlayers];
	eClassTypes			fMemberSmallHulls[tEntityMgr::kMaxNumberofPlayers];
	eClassTypes			fMemberLargeHulls[tEntityMgr::kMaxNumberofPlayers];
	int32				fMemberStartPositions[tEntityMgr::kMaxNumberofPlayers];
	tString				fMemberFleetNames[tEntityMgr::kMaxNumberofPlayers][3];
	tString				fMemberFleetCallNames[tEntityMgr::kMaxNumberofPlayers][3];
	iTruth				fHosting;

	eGameStatus			fCampaignStatus;

	int32				fScriptValues[kMaxScriptValues];

	//----------------------------------------
	// members filled in by script
	//----------------------------------------

	int32				fTotalShipsInGame;
	tShipInGame			fAllShips[60];

	eRaceName			fAlliedRace;
	eRaceName			fEnemyRace;
	int32				fEnemyPrimaryShipIndex;
	int32				fTaggedShipID;			// used by kTagShipRace, kTagShipHull, kTagShipClass...
	int32				fEndGamePauseTime;

	const char*			fMapName;
	const char*			fMissionTitle;
	int32				fMapPicked;
	eSystems			fSystemName;
	int32				fEnableMenuButtons;		// uses eMenuButtons values
	int32				fTechnicalSuccess;
};

#endif	// #ifndef scriptstructs_h
