//	<Name>	tbag.h
//
//     <Legal>     Copyright (c) 1997-1999 Quicksilver Software, Inc.
//     Permission is granted to use this file solely in accordance with the terms and
//     conditions in the SFC Mission API Software Use Limitations and Limited License.
//     All other rights reserved.
//
//     <Module>     Star Trek: Starfleet Command Mission API
//
//
//		

#ifndef TBAGH
#define TBAGH 1





//--------------------------------------------------------------------------------
// Other Headers 

	// Tiger Includes:

		// When including headers, Tiger components should be first!
		
		// tnative.h includes platform specific headers: (as some may be
		//   precompiled, it must come before any other, even before
		//   tlearn.h) Note: It handles the max/min problems when including windows.h

		#include "tString.h"


	// Declaring Tiger Parts
		//   Note: These variables sould not be directly accessed in
		//   code but instead accessed via BindPart through a macro.
		//   example:  #define aClockPart (BindPart(tClockPart))

		// (no parts to declare here, this is a header)


	// Not the Tiger:

		#include <string.h>




//--------------------------------------------------------------------------------
// tBag Support

												//tBag and tString are interdependent
												// though philisophicaly seperate entities.
class tString;

#ifndef byte
	typedef unsigned char byte;
#endif


typedef struct
{
												//History of total constructors and destructors called
	short fTotalConstructorsCalled;
	short fTotalDestructorsCalled;
												//Memory tracking variables.
	long fTotalBagSize;
	long fLargestTotalBagSize;

} sMasterBagControlType;




//--------------------------------------------------------------------------------
// tBaseItemBag & items

typedef enum
{
	kNoBaseBagType = 0,
	kByteBag,
	kLongBag,
	kCharArrayBag,
	kTStringBag,
	kNumBaseBagTypes
} eBaseBagTypes;




												// For storing items 

class tBaseItemBag;
class tBaseItemBag
{
	public:
												//What type of bag is this.
	int8			fBagType;
												//Char array id
	char 			*fLabel;
												//Next item in linked list of bag items
	tBaseItemBag	*fNext;

												//Memory tracking for all of tItemBag
	static sMasterBagControlType 	smMasterControl;
	
												// initializations.
	static void mInitMasterControl(void);
	
												//Updating memory info
	static void mUpdateMasterMemInfo(long delta);
	static void mUpdateMasterConstructorInfo(long delta);
	static void mUpdateMasterDestructorInfo(long delta);


												//info on state of memory
	static long mTotalMemSize(void);
	static long mLargestMemSize(void);
	static long mTotalConstructors(void);
	static long mTotalDestructors(void);


												//All heap memory for tItemBag uses these
	static byte* 	mGetMemory(long size);
	static void 	mFreeMemory(void *ptr,long id = 0);
	

												//new overload for memory management.
	void *operator new(size_t size);
	void operator delete(void *ptr);

	tBaseItemBag();
	virtual ~tBaseItemBag();
};






												//Storing byte items
class tByteItemBag:public tBaseItemBag
{
	public:
		byte *fData;

	tByteItemBag(byte initalizer);
	virtual ~tByteItemBag();
};


												//Storing long items.
class tLongItemBag:public tBaseItemBag
{
	public:
		long *fData;

	tLongItemBag(long initalizer);
	virtual ~tLongItemBag();
};


												//Storing char arrays.
class tCharArrayItemBag:public tBaseItemBag
{
	public:
		char *fData;

	tCharArrayItemBag(char *initalizer);
	virtual ~tCharArrayItemBag();
};


												//Storing tStrings
class tTStringItemBag:public tBaseItemBag
{
	public:
		tString *fData;

	tTStringItemBag(tString &initalizer);
	virtual ~tTStringItemBag();
};





//--------------------------------------------------------------------------------
// tItemBag


												// User interface for storing items 
class tItemBag
{
	protected:		// Make these protected to allow inheritance  4/2/98
		long			fNumItems;
		tBaseItemBag 	*fBagItemList;

		long 			mAddToList(tBaseItemBag *Item, char *label);
		tBaseItemBag 	*mFindItemWithLabel(char *label);
		long 			mRemoveItemWithLabel(char *label);
	
	public:
		
												//General info about the class state
		static long mTotalMemSize(void);
		static long mLargestMemSize(void);
		static long mTotalConstructors(void);
		static long mTotalDestructors(void);


												//storing bytes
		long mAddItem(byte data, char *label);
		long mGetItem(byte *dest,char *label);


												//storing longs
		long mAddItem(long data, char *label);
		long mGetItem(long *dest,char *label);


												//storing char arrays
		long mAddItem(char* data, char *label);
		long mGetItem(char* dest, char *label);


												//storing tStrings;
		long mAddItem(tString& data, char *label);
		long mGetItem(tString& dest, char *label);


		long mDelItem(char *label);



		void operator=(tItemBag &b);

		void *operator new(size_t size);
		void operator delete(void *ptr);

												// special memory cheking constructor.
		tItemBag(long code);
		
		tItemBag();
		virtual ~tItemBag();
};

#endif
