//	<Name>	team.h
//
//     <Legal>     Copyright (c) 1997-1999 Quicksilver Software, Inc.
//     Permission is granted to use this file solely in accordance with the terms and
//     conditions in the SFC Mission API Software Use Limitations and Limited License.
//     All other rights reserved.
//
//     <Module>     Star Trek: Starfleet Command Mission API


#ifndef team_h
#define team_h




#include "entitymgr.h"
#include "eracename.h"



// these two enums are also used in script modules
enum eTeamID { kNoTeam = -1,kTeam1 = 0,kTeam2,kTeam3,kTeam4,kTeam5,kTeam6,kTeam7,kTeam8,kTeam9,kTeam10,
	kTeam11,kTeam12,kTeam13,kTeam14,kTeam15,kTeam16,kTeam17,kTeam18,kTeam19,kTeam20,kMaxTeams,kForceToInt32  = 0xFFFFFFFF};

enum eTeamRelationship
{
	kUnknownRelationship	= 0,	// do not use in script
	kTeamSelfAware			= 1,	// this is the relationship of a team to itself - do not use in script

	kTeamAllied				= 2,
	kTeamFriendly			= 3,
	kTeamNeutral			= 4,
	kTeamHostile			= 5,
	kTeamAtWar				= 6,

	kTeamPassive			= 7		// this is the relationship of a terrain feature (sun, asteroid, etc...) to a team
									// do not use in script
};


enum eTeamColor {	kFederationPrime = kFederation, kKlingonPrime = kKlingon,
					kRomulanPrime= kRomulan,	kLyranPrime = kLyran,
					kHydranPrime= kHydran,kGornPrime= kGorn,
					kOrionPrime= kOrion,	kOtherPrime,
					kFederationAlt = kFederation + kOtherPrime , kKlingonAlt = kKlingon + kOtherPrime,
					kRomulanAlt= kRomulan + kOtherPrime,kLyranAlt= kLyran + kOtherPrime,
					kHydranAlt= kHydran + kOtherPrime,kGornAlt = kGorn + kOtherPrime,
					kOrionAlt= kOrion + kOtherPrime,kOtherAlt };

enum eTeamDamagetypes
{
	kDamageTypeBPVLose,
	kDamageTypeInternal,
	kDamageTypeShields,
	kDamageTypeCombinedInternalAndShields,
};

typedef xvector(tID) tTeamMemberContainer;

class tEntity;
class tAIAdmiral;
class tBasicTeam;

#define kKilledEntityListSize 20

// do not add any functions of any sort to this
struct tWounded
{
	int32 fEntityID;
	int32 fHitsRemaining;
};
class tBasicTeamMgr
{
public:
	tBasicTeamMgr();
	~tBasicTeamMgr();

	void mBeginGame();
	void mEndGame();

	//A return of kNoTeam means there are no more team slots available
	eTeamID mAddNewTeam(eRaceName AssociatedRace = kFederation);
	tBasicTeam *mGetTeam(eTeamID teamID);
	eTeamID mGetTeamID( tID entityID );

	eTeamRelationship mGetRelationshipBetweenTeams(eTeamID teamID1, eTeamID teamID2);
	eTeamRelationship mGetRelationshipBetweenEntities( tID entity1ID, tID entity2ID );
	void mSetTeamRelationship(eTeamID teamID1, eTeamID teamID2, eTeamRelationship relationship);

	// this function hopes to prevent two weapons on the same team killing the same entity
	iTruth mHasThisEntityAlreadyBeenKilledByOurTeam(int32 entityID, eTeamID teamID);
	void mAddEntityToKilledListOfOurTeam(int32 entityID, eTeamID teamID);
	eTeamRelationship mGetRelationshipOfThisEntityToOurTeam(tEntity *entity, eTeamID teamID);

	// returns ID of team that matches the given relationship for the given team
	// (mGetTeamWithRelations will create one if one does not exist)
	eTeamID mFindTeamWithRelations (eTeamID teamID, eTeamRelationship Relations);
	eTeamID mGetTeamWithRelations (eTeamID teamID, eTeamRelationship Relations);
	void mAddToTotalDamage (eTeamID teamID, int32 Damage, eTeamDamagetypes Type);
	int32 mGetTotalDamage (eTeamID teamID, eTeamRelationship RelationshipToTeam, eTeamDamagetypes Type);


	tWounded *mGetWoundedList(eTeamID teamID);
	int32 mGetSizeOfWoundedList(eTeamID teamID);
	void mAddToWoundedList(eTeamID teamID, int32 ID, int32 hitsRemaining);
	void mRemoveFromWoundedList(eTeamID teamID, int32 ID);
	iTruth mAreEntitiesTeamMates (tID Entity1, tID Entity2);
protected:
	void mCleanUpTeams();
	iTruth fGameStarted;
	tBasicTeam*	fTeams[kMaxTeams];
};
class tBasicTeam
{
public:
	tBasicTeam( eTeamID teamID );
	~tBasicTeam();

	void mAddEntityToTeam( int32 shipID );
	void mRemoveEntityFromTeam( int32 shipID );
	iTruth mIsEntityOnTeam (int32 shipID);

	eTeamRelationship mGetRelationshipWithTeam(eTeamID teamID) { return fRelationships[teamID]; };
	void mSetTeamRelationship(eTeamID teamID, eTeamRelationship relationship);

	// this function hopes to prevent two ESGs on the same team killing the same entity
	iTruth mHasThisEntityAlreadyBeenKilledByOurTeam(int32 ID);
	void mAddEntityToKilledList(int32 ID);

	tWounded *mGetWoundedList() { mCleanUpWoundedList(); return fWoundedList; };
	int32 mGetSizeOfWoundedList() { return fNumberOnWoundedList; };
	void mAddToWoundedList(int32 ID, int32 hitsRemaining);
	void mRemoveFromWoundedList(int32 ID);

	eTeamID mGetTeamID();

	tTeamMemberContainer* mGetListOfTeamMateIDs (void) {return &fTeamMateIDs;};
		
	tTeamMemberContainer fTeamMateIDs;
	// Also sets the default color.
	void mSetRaceAssociation(eRaceName race) {fAssociatedRace = race;fAssociatedColor = (eTeamColor)race; };
	eRaceName mGetRaceAssociation(void) { return fAssociatedRace;};
	void mSetColorAssociation(eTeamColor color) {fAssociatedColor = color;};
	eTeamColor mGetColorAssociation(void) { return fAssociatedColor;};

	void mAddToTotalDamage (int32 Damage, eTeamDamagetypes Type);
	int32 mGetTotalDamage (eTeamDamagetypes Type);
protected:
	//Accessors for the AI
	friend class tAIManager;
	tID mGetAdmiralID();
	void mSetAdmiralID( tID admiralID );
	tID mRemoveAdmiral();

	eTeamRelationship fRelationships[kMaxTeams];

	// ESG and things need to keep lists of things they think they've destroyed --this is currently
	//	where we will keep this information...
	int32 fKilledIDList[kKilledEntityListSize];
	int32 fNumberOnKilledIDList;
	int32 fLastKilledIDAdded;
	int32 fTotalDamage[kDamageTypeCombinedInternalAndShields];
	
	tWounded *fWoundedList;
	int32 fNumberOnWoundedList;
	int32 fMaxNumberOnWoundedList;

	eTeamID fTeamID;
	tID fTeamAdmiralID; //The Ai admiral in charge of this team
	eRaceName fAssociatedRace;
	eTeamColor fAssociatedColor;
	void mCleanUpWoundedList();

};



extern tBasicTeamMgr *gBasicTeamMgr;

#endif
