//	<Name>	
//
//     <Legal>     Copyright (c) 1997-1999 Quicksilver Software, Inc.
//     Permission is granted to use this file solely in accordance with the terms and
//     conditions in the SFC Mission API Software Use Limitations and Limited License.
//     All other rights reserved.
//
//     <Module>     Star Trek: Starfleet Command Mission API
#ifndef tmissionmap_h
#define tmissionmap_h

	// Forward references
class	tMapInfo;
struct  tPoint3;

struct tMapPoint {
	int16	x;
	int16	y;
	} ;

class tMissionMap
{
	public:
	enum{
			kOffMapSpace,
			kNearMapEdgeSpace,
			kNormalSpace,
			kDustCloudLight,
			kDustCloudMedium,
			kDustCloudHeavy,
			kAsteroidLight,
			kAsteroidMedium,
			kAsteroidHeavy,
			kIceLight,
			kIceMedium,
			kIceHeavy,
			kNebulaLight,
			kNebulaHeavy,
			kStar,
			kPlanet,
			kBlackHoleSmall,
			kBlackHoleLarge,
			kFissure,
			kWormHole,
			kCorpse
	};
	enum{	// values cannot go beyond 127
		kMapAsteroid	=  1,				// uses 6 types of asteroids
		kMapDust		= 10,				// uses 3 types of dust clouds
		kMapNebula		= 15,				// uses 2 types of nebulas
		kMapStarts		= 20,				// uses 20 start positions
		kMapEnds		= 45,				// uses 20 end positions
		kMapPlanets		= 70,				// uses 7 types of planets
		kMapStars		= 80,				// uses 3 types of suns
		kMapBlackHoles	= 85,				// uses 2 types of black holes
		kMapFissures	= 90,				// uses 1 type of fissure
		kMapWormHoles	= 95,				// uses 1 type of worm hole
		kMapCorpses		=100,				// uses 1 type of corpse
	};
	
		tMissionMap ( void );
		virtual ~tMissionMap();

					// Methods to build the map (used by mission loader)
		void	mCreateMapObject (unsigned char MapObject, int32 X, int32 Y);
		void	mAddAsteroid ( int32 density, tMapPoint location );	
		void	mAddDustCloud ( int32 density, tMapPoint location );	
		void	mAddEndLoc ( int32 index, tMapPoint location );
		void	mAddNebula ( int32 density, tMapPoint location );	
		void 	mAddPlanet ( int32 index, tMapPoint location );
		void 	mAddStar ( int32 index, tMapPoint location );
		void	mAddStartLoc ( int32 index, tMapPoint location );
		void 	mAddBlackHole ( int32 size, tMapPoint location );
		void	mAddFissure ( int32 size, tMapPoint location );
		void mAddWormHole ( int32 size, tMapPoint location );
		void	mAddTopRightMapBoundary(tMapPoint location);
		void	mAddBotLeftMapBoundary(tMapPoint location);
		void	mAddCorpses ( int32 size, tMapPoint location );
		
					//	Player location information retrieval
		void	mSetMapSize ( int32 rows, int32 cols );
		void	mGetMapSize ( int32* rows, int32* cols );
		int32	mCountStartLocs ( void );
		int32	mCountStartLocs ( int32 playerno );
		tPoint3	mGetEndLoc ( int32 playerno, iTruth* exists );
		tPoint3	mGetStartLoc ( int32 playerno, iTruth* exists );
		tPoint3	mGetStartLoc ( int32 playerno, int32 num, iTruth* exists );
		int		mGetMapData(tPoint3 LocationIn3D);
		int		mGetMapData(int32 X, int32 Y);
		tMapPoint mFindMapEntry (unsigned char MapEntry);
		tMapPoint mGetValidStartPoint (int32 GridX, int32 GridY);

					// Map to screen location conversion
		tPoint3	mConvertLocation ( tMapPoint );
					// Convert map distance to screen distance
		float	mConvertMapDistance (float Distance);
					// screen to map location conversion
		tMapPoint m3DToMapPoint (tPoint3 LocationIn3D);
		
		iTruth	mIsThisABlackHoleMap(void) {return fBlackHoleMap;};
					//returns map width and height
		static tMapPoint mGetMapDimension (void) { return fMapSize;};
		
	protected:
		iTruth  mAreaClear (int32 GridX, int32 GridY);
					// Map to screen location conversion
		tPoint3	mConvertPlanetLocation	( tMapPoint );
		tMapInfo*	fInfo;
		iTruth	fBlackHoleMap;
		static tMapPoint fMapSize;
};

#endif
