//	<Name>	tstring.h

//     <Legal>     Copyright (c) 1997-1999 Quicksilver Software, Inc.
//     Permission is granted to use this file solely in accordance with the terms and
//     conditions in the SFC Mission API Software Use Limitations and Limited License.
//     All other rights reserved.
//
//     <Module>     Star Trek: Starfleet Command Mission API

//	<Info>	tString header

#ifndef tstring_h
#define tstring_h
#pragma once

#include "tlearn.h"
#include "tbasic.h"
#include "tcglue.h"
#include "tigerstl.h"

#include "tBag.h"

	// kMaxStringClassLength comes from the build control,
	// along with kGenericStringImpl.

#include "tstringdefs.h"
#if kUseMemoryPart
	#include "tmemory.h"
#endif


												//tString and tBag are interdependent
												// though philosophicaly separate entities.
class tItemBag;




												//Basic unit of data storage.
												//Should be 2 byte shorts for unicode compatability.
												//Can be other data types if needed.
typedef iWideChar	iBasicStringElement;
	

												// Forward declaration for tString nesting.
class tString;



												// data type for tracking info on a single instance of tString
typedef struct
{
												//Handle for char strings to the outside world to see.
	char			*fTempExternalCharRepresentation;
	int32			fSizeOfExternalCharAllocation;

												//Needed for computations with tString operators.
	tString			*fTempOperatorSpace;

												//Total memory available for data storage (stringSpace).  Can 
												//be larger than data count.
	int32			fMemLimit;
												
												//Size of data space in string (pointed to by stringSpace)
	int32			fDataCount;

												//Pointer to buffer to hold data array.
												//Using STL Vector

	xvector( iBasicStringElement )	fMemSpace;
	
#if 0	// this goo not needed now - rbarris
	#if (CompilerBorland || CompilerMicrosoft)
		// Rogue Wave STL vectors have only one argument
		vector<iBasicStringElement> fMemSpace;
	#else
		vector<iBasicStringElement, allocator<iBasicStringElement> >	fMemSpace;
	#endif
#endif

} tInstanceStringControl;


// Foward declaration of a friend function to make VC 6 happy.
// Since tString has a != operator which is a method function (and not a friend) but it
// uses the == friend operator
//
iTruth operator== (const tString &s1,const tString &s2);
iTruth operator<  (const tString &s1,const tString &s2);

class tString
{
	protected:
												// Variables

												//Specific instance information for an individual tString variable
												//Changed to pointer 4-4-97 to reduce stack thrashing.
		tInstanceStringControl fInstanceControl;


												// tString Memory


												//All memory usage by tString uses these calls
		void*	mGetMemory(int32 size);
		void	mFreeMemory(void* ptr, int32 id=0);


												// data field handling

		void 	mInitInstanceControl(void);


												//routine for generating values for strings that may grow past their
												//inital size.  		
		int32 	mExpansionRoom(const char *s);
		int32 	mExpansionRoom(const iBasicStringElement *s);
		int32 	mExpansionRoom(int32 size);


		
												//Converts an array of iBasicStringElement into stored tString data.
		iTruth 	mIncorporateCharList(const iBasicStringElement *s, int32 lengthToCopy);
		iTruth 	mIncorporateCharList(const iBasicStringElement *s);

												//Converts an array of chars into stored tString data.
		iTruth 	mIncorporateCharList(const char *s, int32 lengthToCopy);
		iTruth 	mIncorporateCharList(const char *s);


												//returns pointer to tString storage
		iBasicStringElement *mDataStr(void);



	public:
												// for needs of an iterator class.
		class 	iterator;
		friend 	iterator;

	
												// constructors and destructors
												
												// Constructors.
		tString(const char *s);						
		tString(const char *s, int32u length);
		tString(const iBasicStringElement *s);						
		tString(const iBasicStringElement c);
		tString(const tString &s);
		tString(const iterator &i);
		tString();
		
												// Destructor
		virtual ~tString();								




												// More Memory


												//Information about a specific instance of tString
		int32 mGetMemSize(void);
		int32 mGetDataCount(void);

												//Preset memory size to avoid excessive memory munching.
		void  mReserve(int32 size);


												// more data field handling

												//Method for generating a character c string representation of
												//the contents of an instance.
		char* mCharStr(void);
		
												//returns pointer to widechar storage
		iBasicStringElement *mWideStr(void);


												//Method for generating a pascal string representation of
												//the contents of an instance (for Mac stuff)
		unsigned char* mPascStr(void);
		

		//
		//	These functions are for retrieving the data of the tString without
		//	causing the allocation of any memory (to be used at interrupt time, etc.
		//
		int			mCopyCharStr( char *str, int max_size );
		int			mCopyWideStr( iWideChar *str, int max_size );
		int			mCopyPascStr( unsigned char *str, int max_size );

												// tString operations

												//Add a string integer to the end of this string
		tString&	mAppendInt32s(int32s val,int32s minDigits = 1);

												//Return the text of the string as an integer
		int32s		mGetInt32s(void);
		
												//Return the text of the string as a float
		float		mGetFloat(void);

												//End the string at indicated location
		void		mEndStrAt(int32 strPosition);
		void		mEndStrAt(iterator strPosition);
		

												//Basic Operators for tString
		void 		operator=  (const char* s);
		
		tString& 	operator=  (const tString &s);
		
		tString& 	operator+  (const tString &s);
		
		tString& 	operator+= (const tString &s);
		tString&	operator+= (const iBasicStringElement &c);
		


		// Needed to make VC5 happy (bsnyder 8/7/97)
		iTruth	operator != (const tString& t) const	
		{ 
			return !(*this == t); 
		}

												//Bool Operations for tString.
												// Note:  These functions are implemented as friends because
												//  stl will use them to derive the full complement of comparitives
												//  using *two* const parameters.
		friend iTruth operator== (const tString &s1,const tString &s2);
		friend iTruth operator<  (const tString &s1,const tString &s2);
		friend iTruth operator>  (const tString &s1,const tString &s2);
		// Added to make VC5 happy.
		friend iTruth operator<= (const tString &s1,const tString &s2);
		friend iTruth operator>= (const tString &s1,const tString &s2);



												//Bool Operations for tString and char c strings. Implemented as 
												// friends so both params can be const.
		friend int16 mCStrCompare(const tString &s1, const char* s2);

		friend iTruth operator== (const tString &s1,const char *s2);
		friend iTruth operator>  (const tString &s1,const char *s2);
		friend iTruth operator<  (const tString &s1,const char *s2);
		friend iTruth operator<= (const tString &s1,const char *s2);
		friend iTruth operator>= (const tString &s1,const char *s2);
		friend iTruth operator!= (const tString &s1,const char *s2);

		// (mig 03/07/97)
		//
		//	Simulation of needed <string.h> functions.	(Add to these as people need them)
		//
		iterator				mStrChr(iBasicStringElement c);
		iterator				mStrRChr(iBasicStringElement c);
		int						mStrLen(void) const;
		void					mToUpper(void);
		void					mToLower(void);
		int32s					mStrICmp(const tString &s) const;
		int32s					mStrCmp(const tString &s) const;
		int32s					mStringCompare(const tString &s) const; // The root that the other two call.
		//
		//	Bracket dereference operator
		//
//		iterator				operator[](int n);


		//
		//	char * additive operators
		//
		tString& operator+ (const char *s);
		tString& operator+= (const char *s);


		//
		//	atomic unit operators
		//
		void operator= (const iBasicStringElement* s);
		void operator= (const unsigned char* s);		// for mac compatibility, MUST BE C STRING!


		//	(mig 03/25/97)
		//
		//	Iterator support functions
		//
		iterator				begin();
		iterator				end();

		// gshackel 16Apr97
		// new functions (esp. the substring!)
		//
		tString 				substr(int32 start, int32 length);
		tString 				substr(int32 start);
		iBasicStringElement&	operator[](int32 n);

		int32 					size() const;

		//
		// basic string search functions
		//
		int32					find(char);
		int32					find(tString&, int32 startPos = 0);
		int32					rfind(tString&, int32 startPos);
		int32					find(const char *, int32 startPos = 0);
		int32					rfind(const char *, int32 startPos);
		int32					rfind(const char *);


		// native string conversion (OS specific string)
		#if	SystemWin32
			char*				mOSString();
			tString*			mAppendOSStr(char*	osStr);	
		#elif SystemMac7
			unsigned char*		mOSString();
			tString*			mAppendOSStr(unsigned char*	osStr);
		#endif

		
		#if kUnicodeConvert
			// For UTF8 Conversion

			// Convert tString to UTF8 string
			int8u*		mUTF8Str();

			//	Append UTF8 string to this tString
			tString*	mAppendUTF8Str (const int8u*	UTF8Str);
		#endif
	
};




#ifndef LESS_SS
#define LESS_SS
class less_ss : public binary_function<const tString&, const tString&, bool>
{	public:
	bool operator () (const tString& x, const tString& y)
		const 	{	return (x.mStrCmp(y) < 0); }
};

class less_ps : public binary_function<const tString*, const tString*, bool>
{	public:
	bool operator () (const tString* x, const tString* y)
		const 	{	return ( (*x).mStrCmp( (*y) ) < 0); }
};
#endif







// mig (03/25/97)
// The scoped iterator class
//
//	These things get passed on the stack (12 bytes), so we don't ever have to
//	track data allocation for them (they live and die in scope).
//
//
class	tString::iterator
{
	friend tString;

	protected:
		int32		fIndex;			// the index value to which we now "point"
		tString		*fString;		// the string for which we are an iterator

	public:

		// Constructors
		iterator();

		iterator(const tString::iterator &i);

		iterator(int32u index, tString* string);

		// basic bracket dereference
		iBasicStringElement & operator[](int32 n);

		// basic asterisk dereference
		iBasicStringElement & operator * ();

		// assignment equals
		iBasicStringElement operator = (const iBasicStringElement c);

		// addition
		tString::iterator operator + (int32 n);

		// subtraction
		tString::iterator operator - (int32 n);

		// pre increment
		tString::iterator operator ++ ();

		// post increment?
		tString::iterator operator ++ (int);

		// pre decrement
		tString::iterator operator -- ();

		// post decrement?
		tString::iterator operator -- (int);

		// comparative equals
		iTruth operator == (const tString::iterator i);

		// comparative equals
		iTruth operator == (const iBasicStringElement c);

		// comparative !equals
		iTruth operator != (const tString::iterator i);

		// comparative !equals
		iTruth operator != (const iBasicStringElement c);

		// comparative less than
		iTruth operator < (const tString::iterator i);

		// comparative less than
		iTruth operator < (const iBasicStringElement c);

		iTruth operator > (const iBasicStringElement c);

		iTruth operator <= (const iBasicStringElement c);
		
		iTruth operator >= (const iBasicStringElement c);
};

//
//	Extra little tidbits.  Needed for "mAppendInt32s" method.
//
int32 StringToNum(char *str);
void NumToString(tString &destStr, int32 data, int32s minDigits = 1);

#endif // _TSTRING_H_

