//	<Name>	tstringdefs.h

//     <Legal>     Copyright (c) 1997-1999 Quicksilver Software, Inc.
//     Permission is granted to use this file solely in accordance with the terms and
//     conditions in the SFC Mission API Software Use Limitations and Limited License.
//     All other rights reserved.
//
//     <Module>     Star Trek: Starfleet Command Mission API

//	Default values

	//How many basic units an instance will store.  Mainly used for
	//sanity purposes.  Can not be larger than an unsigned short int.
#define kMaxStringClassLength	 50000			

//
//	Here's a keen little thing.
//
//	If you replace code written to be very "generic"
//	(ie: not dependant upon the real internals of the object)
//	with code that's specific to the object's internals (ie: "much faster"),
//	you should wrap the old code in this #if.  That way,
//	it can be set back to the old way if the internals get
//	changed and then someone can come along and look for the
//	all of these places and replace the old specific code with
//	new specific code.
//
//	NOTE: This doesn't imply that the only implementation
//	specific code within the object will be wrapped in this.
//	There's _LOTS_ of that in the code already.  It's just a
//	nice thing to do from now on, to keep people happy and
//	make it easier for us to be able to change the internals
//	in the future if we need them.
//
//	NOTE: IF YOU SET THE GENERIC CODE ON, YOU'D BETTER HAVE A
//	VERY GOOD REASON.  Being thoroughly generic is all well and
//	good, but it also often tends to be FRIGHTFULLY slow.
//  I'm setting this up so we can keep our code to be at least
//	partially quick, while still being able to fall back on the
//	old slow ways WHEN WE CHANGE IMPLEMENTATION, but NOT
//	because we simply feel like it.
//
//
#define kGenericStringImpl  (0)


//
//	Ensabling this #define will cause the string's internal
//	memory allocators to use tMemeory instead of basic malloc.
//
//	The default is 'off' in order to avoid having an impact on
//	the current code base which may not be including the tMemory
//	code.
//
#define kUseMemoryPart		(0)												


//	Enable this #define to use the conversion functions
//  (mUTF8Str(), and mAppendUTF8Str(int8u*)
// 
#define kUnicodeConvert		(0)


//	Optional overrides 
#ifdef	tstringNonDefault
	#include	"tstringopts.h"	//client supplied header
#endif

//
//	This complex thing goes AFTER the tstringopts.h inclusion,
//	because it requres testing kUseMemoryPart, which could
//	be overridden by tstringopts.h
//
#ifndef kDebugMemoryUsage
	#ifdef _DEBUG
		#if kUseMemoryPart
			#define kDebugMemoryUsage	(1)
		#else
			#define kDebugMemoryUsage	(0)
		#endif
	#else
		#define kDebugMemoryUsage	(0)
	#endif
#endif

