// keeps track of teams in multiplayer games
class tTeamTrackerClass
{
public:
		void			mInitialize (eTeamID LocalTeam, tDynaverseScriptInfo& info, int32 OverrideShipCounts);
		void			mShipDied (eTeamID Team);
		void			mTeamGone (eTeamID Team, tMissionInfo* Mission);	// call with mission ptr to enable goal indicator
		eGameStatus		mGetTeamsStatus (void);
		void			mTrackLeader (tMissionInfo* Mission);

protected:
		eTeamID			fLocalPlayerTeam;
		int32			fStartingShipCounts[kReserved+1];
		int32			fTeamShipCounts[kReserved+1];
		iTruth			fTeamIsHuman[kReserved+1];
		int32			fMultiplayerTeams[kReserved+1];
};

//---------------------------------------------------------------------------------------------
//---------------------------------------------------------------------------------------------
// init all protected params
void tTeamTrackerClass::mInitialize (eTeamID LocalTeam, tDynaverseScriptInfo& info, int32 OverrideShipCounts)
{
	fLocalPlayerTeam = LocalTeam;
	for (int32 i = 0; i <= kReserved; i++)
	{
		fStartingShipCounts[i] = 0;
		fTeamShipCounts[i] = 0;
		fTeamIsHuman[i] = kFalse;
		fMultiplayerTeams[i] = -1;
	}

	// fill in arrays based on given info
	for (i = 0; i <= kReserved; i++)
	{
		if (info.fMembersInGame[i] != kPlayerTypeNoOne)
		{
			if (OverrideShipCounts)
			{
				fStartingShipCounts[i] = OverrideShipCounts;
			}
			else
			{
				fStartingShipCounts[i] = info.fMemberShipCounts[i];
			}
			fTeamShipCounts[i] = fStartingShipCounts[i];
			fMultiplayerTeams[i] = info.fMemberTeams[i];
			if (info.fMembersInGame[i] == kPlayerTypeHuman)
			{
				fTeamIsHuman[i] = kTrue;
			}
		}
	}
}

//---------------------------------------------------------------------------------------------
//
void tTeamTrackerClass::mShipDied (eTeamID Team)
{
	if (Team <= kReserved)
	{
		fTeamShipCounts[Team]--;
	}
}

//---------------------------------------------------------------------------------------------
//
void tTeamTrackerClass::mTeamGone (eTeamID Team, tMissionInfo* Mission)
{
	if (Team <= kReserved)
	{
		fTeamShipCounts[Team] = 0;
		if (Mission)
		{
			mTrackLeader (Mission);
		}
	}
}

//---------------------------------------------------------------------------------------------
// every time a ship dies we need to determine if there is one lone leader
// that the others need to gang up on. If a leader is found then the
// script will turn on the goal indicator for its local player.
void tTeamTrackerClass::mTrackLeader (tMissionInfo* Mission)
{
	// turn off the indicator
	Mission->mSetGoalIndicator (kFalse, 0, 0);

	// determine which team is the leader
	int32 leader;
	int32 leader_ship_count = 0;
	iTruth new_leader_found = kFalse;
	for (int32 i = 0; i <= kReserved; i++)
	{
		if (fTeamShipCounts[i] >= leader_ship_count)
		{
			new_leader_found = (fTeamShipCounts[i] > leader_ship_count);
			leader_ship_count = fTeamShipCounts[i];
			leader = i;
		}
	}

	// if we did not find a new leader then set leader flag to local team.
	// this will force the goal indicator off for everyone
	if (new_leader_found == kFalse)
	{
		leader = fLocalPlayerTeam;
	}

	if (leader != fLocalPlayerTeam && leader_ship_count > 0)
	{
		// found a new leader and its not the local team - turn on indicator
		tTeamInfo* team = Mission->mGetTeamHandle (leader);
		if (team)
		{
			tShipInfo* Ship = team->mGetThisShip ((fStartingShipCounts[leader] - fTeamShipCounts[leader]) + 1);
			if (Ship)
			{
				Mission->mSetGoalIndicator (Ship);
			}
		}
	}
}

//---------------------------------------------------------------------------------------------
// gets status of game
//		returns kPlayerWins if local player on the last team alive
//		returns kPlayerLoses if the only team left alive is not the local player or only AI teams alive
//		returns kGameInProgress if still more teams in play
eGameStatus tTeamTrackerClass::mGetTeamsStatus (void)
{
	int32 human_teams_count = 0;
	int32 hostile_teams_count = 0;
	int32 first_team_alive = -1;

	// count the number of hostile and human teams
	for (int32 i = 0; i <= kReserved; i++)
	{
		if (fTeamShipCounts[i] > 0)
		{
			if (first_team_alive == -1)
			{
				first_team_alive = fMultiplayerTeams[i];
			}
			if (fMultiplayerTeams[i] != first_team_alive)
			{
				hostile_teams_count++;
			}
			if (fTeamIsHuman[i])
			{
				human_teams_count++;
			}
		}
	}

	// local player loses if no humans alive
	if (human_teams_count == 0)
	{
		return kPlayerLoses;
	}

	// game continues if any hostile teams are left
	if (hostile_teams_count > 0)
	{
		return kGameInProgress;
	}

	// local player wins if he's on the team that's still alive
	if (fMultiplayerTeams[fLocalPlayerTeam] == first_team_alive)
	{
		return kPlayerWins;
	}

	// he's a loser
	return kPlayerLoses;
}

